# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/5 17:00
Desc: 巨潮资讯-个股-历史分红
https://webapi.cninfo.com.cn/#/company?companyid=600009
"""

import pandas as pd
import requests
import py_mini_racer

from akshare.datasets import get_ths_js


def _get_file_content_ths(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_dividend_cninfo(symbol: str = "600009") -> pd.DataFrame:
    """
    巨潮资讯-个股-历史分红
    https://webapi.cninfo.com.cn/#/company?companyid=600009
    :param symbol: 股票代码
    :type symbol: str
    :return: 历史分红
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1139"
    params = {"scode": symbol}
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "实施方案公告日期",
        "送股比例",
        "转增比例",
        "派息比例",
        "股权登记日",
        "除权日",
        "派息日",
        "股份到账日",
        "实施方案分红说明",
        "分红类型",
        "报告时间",
    ]
    temp_df["实施方案公告日期"] = pd.to_datetime(
        temp_df["实施方案公告日期"], errors="coerce"
    ).dt.date
    temp_df["送股比例"] = pd.to_numeric(temp_df["送股比例"], errors="coerce")
    temp_df["转增比例"] = pd.to_numeric(temp_df["转增比例"], errors="coerce")
    temp_df["派息比例"] = pd.to_numeric(temp_df["派息比例"], errors="coerce")
    temp_df["股权登记日"] = pd.to_datetime(
        temp_df["股权登记日"], errors="coerce"
    ).dt.date
    temp_df["除权日"] = pd.to_datetime(temp_df["除权日"], errors="coerce").dt.date
    temp_df["派息日"] = pd.to_datetime(temp_df["派息日"], errors="coerce").dt.date
    temp_df.sort_values(by="实施方案公告日期", ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    stock_dividend_cninfo_df = stock_dividend_cninfo(symbol="600009")
    print(stock_dividend_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/10 18:30
Desc: 股票指数数据-新浪-东财-腾讯
所有指数-实时行情数据和历史行情数据
https://finance.sina.com.cn/realstock/company/sz399552/nc.shtml
"""

import datetime
import re

import pandas as pd
import py_mini_racer
import requests

from akshare.index.cons import (
    zh_sina_index_stock_payload,
    zh_sina_index_stock_url,
    zh_sina_index_stock_count_url,
    zh_sina_index_stock_hist_url,
)
from akshare.stock.cons import hk_js_decode
from akshare.utils import demjson
from akshare.utils.func import fetch_paginated_data
from akshare.utils.tqdm import get_tqdm


def _replace_comma(x):
    """
    去除单元格中的 ","
    :param x: 单元格元素
    :type x: str
    :return: 处理后的值或原值
    :rtype: str
    """
    if "," in str(x):
        return str(x).replace(",", "")
    else:
        return x


def get_zh_index_page_count() -> int:
    """
    指数的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hs_s
    :return: 需要抓取的指数的总页数
    :rtype: int
    """
    res = requests.get(zh_sina_index_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_zh_index_spot_sina() -> pd.DataFrame:
    """
    新浪财经-行情中心首页-A股-分类-所有指数
    大量采集会被目标网站服务器封禁 IP, 如果被封禁 IP, 请 10 分钟后再试
    https://vip.stock.finance.sina.com.cn/mkt/#hs_s
    :return: 所有指数的实时行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = get_zh_index_page_count()
    zh_sina_stock_payload_copy = zh_sina_index_stock_payload.copy()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_count + 1), leave=False):
        zh_sina_stock_payload_copy.update({"page": page})
        res = requests.get(zh_sina_index_stock_url, params=zh_sina_stock_payload_copy)
        data_json = demjson.decode(res.text)
        big_df = pd.concat(objs=[big_df, pd.DataFrame(data_json)], ignore_index=True)
    big_df = big_df.map(_replace_comma)
    big_df["trade"] = pd.to_numeric(big_df["trade"], errors="coerce")
    big_df["pricechange"] = pd.to_numeric(big_df["pricechange"], errors="coerce")
    big_df["changepercent"] = pd.to_numeric(big_df["changepercent"], errors="coerce")
    big_df["buy"] = pd.to_numeric(big_df["buy"], errors="coerce")
    big_df["sell"] = pd.to_numeric(big_df["sell"], errors="coerce")
    big_df["settlement"] = pd.to_numeric(big_df["settlement"], errors="coerce")
    big_df["open"] = pd.to_numeric(big_df["open"], errors="coerce")
    big_df["high"] = pd.to_numeric(big_df["high"], errors="coerce")
    big_df["low"] = pd.to_numeric(big_df["low"], errors="coerce")
    big_df.columns = [
        "代码",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "_",
        "_",
        "昨收",
        "今开",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
        ]
    ]
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌额"] = pd.to_numeric(big_df["涨跌额"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["昨收"] = pd.to_numeric(big_df["昨收"], errors="coerce")
    big_df["今开"] = pd.to_numeric(big_df["今开"], errors="coerce")
    big_df["最高"] = pd.to_numeric(big_df["最高"], errors="coerce")
    big_df["最低"] = pd.to_numeric(big_df["最低"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    return big_df


def __stock_zh_main_spot_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深重要指数
    https://quote.eastmoney.com/center/hszs.html
    :return: 指数的实时行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://33.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "dect": "1",
        "wbp2u": "|0|0|0|web",
        "fid": "",
        "fs": "b:MK0010",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,"
        "f23,f24,f25,f26,f22,f11,f62,f128,f136,f115,f152",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"].astype(int) + 1
    temp_df.rename(
        columns={
            "index": "序号",
            "f2": "最新价",
            "f3": "涨跌幅",
            "f4": "涨跌额",
            "f5": "成交量",
            "f6": "成交额",
            "f7": "振幅",
            "f10": "量比",
            "f12": "代码",
            "f14": "名称",
            "f15": "最高",
            "f16": "最低",
            "f17": "今开",
            "f18": "昨收",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    return temp_df


def stock_zh_index_spot_em(symbol: str = "上证系列指数") -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深京指数
    https://quote.eastmoney.com/center/gridlist.html#index_sz
    :param symbol: "上证系列指数"; choice of {"沪深重要指数", "上证系列指数", "深证系列指数", "指数成份", "中证系列指数"}
    :type symbol: str
    :return: 指数的实时行情数据
    :rtype: pandas.DataFrame
    """
    if symbol == "沪深重要指数":
        return __stock_zh_main_spot_em()

    url = "https://48.push2.eastmoney.com/api/qt/clist/get"
    symbol_map = {
        "上证系列指数": "m:1+t:1",
        "深证系列指数": "m:0 t:5",
        "指数成份": "m:1+s:3,m:0+t:5",
        "中证系列指数": "m:2",
    }
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "wbp2u": "|0|0|0|web",
        "fid": "f12",
        "fs": symbol_map[symbol],
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,"
        "f26,f22,f33,f11,f62,f128,f136,f115,f152",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.rename(
        columns={
            "index": "序号",
            "f2": "最新价",
            "f3": "涨跌幅",
            "f4": "涨跌额",
            "f5": "成交量",
            "f6": "成交额",
            "f7": "振幅",
            "f10": "量比",
            "f12": "代码",
            "f14": "名称",
            "f15": "最高",
            "f16": "最低",
            "f17": "今开",
            "f18": "昨收",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    return temp_df


def stock_zh_index_daily(symbol: str = "sh000922") -> pd.DataFrame:
    """
    新浪财经-指数-历史行情数据, 大量抓取容易封 IP
    https://finance.sina.com.cn/realstock/company/sh000909/nc.shtml
    :param symbol: sz399998, 指定指数代码
    :type symbol: str
    :return: 历史行情数据
    :rtype: pandas.DataFrame
    """
    params = {"d": "2020_2_4"}
    res = requests.get(zh_sina_index_stock_hist_url.format(symbol), params=params)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", res.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    temp_df = pd.DataFrame(dict_list)
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    return temp_df


def get_tx_start_year(symbol: str = "sh000919") -> pd.DataFrame:
    """
    腾讯证券-获取所有股票数据的第一天, 注意这个数据是腾讯证券的历史数据第一天
    https://gu.qq.com/sh000919/zs
    :param symbol: 带市场标识的股票代码
    :type symbol: str
    :return: 开始日期
    :rtype: pandas.DataFrame
    """
    url = "https://web.ifzq.gtimg.cn/other/klineweb/klineWeb/weekTrends"
    params = {
        "code": symbol,
        "type": "qfq",
        "_var": "trend_qfq",
        "r": "0.3506048543943414",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    if not demjson.decode(data_text[data_text.find("={") + 1 :])["data"]:
        url = "https://proxy.finance.qq.com/ifzqgtimg/appstock/app/newfqkline/get"
        params = {
            "_var": "kline_dayqfq",
            "param": f"{symbol},day,,,320,qfq",
            "r": "0.751892490072597",
        }
        r = requests.get(url, params=params)
        data_text = r.text
        start_date = demjson.decode(data_text[data_text.find("={") + 1 :])["data"][
            symbol
        ]["day"][0][0]
        return start_date
    start_date = demjson.decode(data_text[data_text.find("={") + 1 :])["data"][0][0]
    return start_date


def stock_zh_index_daily_tx(symbol: str = "sz980017") -> pd.DataFrame:
    """
    腾讯证券-日频-股票或者指数历史数据
    作为 ak.stock_zh_index_daily() 的补充, 因为在新浪中有部分指数数据缺失
    注意都是: 前复权, 不同网站复权方式不同, 不可混用数据
    https://gu.qq.com/sh000919/zs
    :param symbol: 带市场标识的股票或者指数代码
    :type symbol: str
    :return: 前复权的股票和指数数据
    :rtype: pandas.DataFrame
    """
    start_date = get_tx_start_year(symbol=symbol)
    url = "https://proxy.finance.qq.com/ifzqgtimg/appstock/app/newfqkline/get"
    range_start = int(start_date.split("-")[0])
    range_end = datetime.date.today().year + 1
    temp_df = pd.DataFrame()
    tqdm = get_tqdm()
    for year in tqdm(range(range_start, range_end), leave=False):
        params = {
            "_var": "kline_dayqfq",
            "param": f"{symbol},day,{year}-01-01,{year + 1}-12-31,640,qfq",
            "r": "0.8205512681390605",
        }
        res = requests.get(url, params=params)
        text = res.text
        try:
            inner_temp_df = pd.DataFrame(
                demjson.decode(text[text.find("={") + 1 :])["data"][symbol]["day"]
            )
        except:  # noqa: E722
            inner_temp_df = pd.DataFrame(
                demjson.decode(text[text.find("={") + 1 :])["data"][symbol]["qfqday"]
            )
        temp_df = pd.concat(objs=[temp_df, inner_temp_df], ignore_index=True)
    if temp_df.shape[1] == 6:
        temp_df.columns = ["date", "open", "close", "high", "low", "amount"]
    else:
        temp_df = temp_df.iloc[:, :6]
        temp_df.columns = ["date", "open", "close", "high", "low", "amount"]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["amount"] = pd.to_numeric(temp_df["amount"], errors="coerce")
    temp_df.drop_duplicates(inplace=True, ignore_index=True)
    return temp_df


def stock_zh_index_daily_em(
    symbol: str = "csi931151",
    start_date: str = "19900101",
    end_date: str = "20500101",
) -> pd.DataFrame:
    """
    东方财富网-股票指数数据
    https://quote.eastmoney.com/center/hszs.html
    :param symbol: 带市场标识的指数代码; sz: 深交所, sh: 上交所, csi: 中信指数 + id(000905)
    :type symbol: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 指数数据
    :rtype: pandas.DataFrame
    """
    market_map = {"sz": "0", "sh": "1", "csi": "2"}
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    if symbol.find("sz") != -1:
        secid = "{}.{}".format(market_map["sz"], symbol.replace("sz", ""))
    elif symbol.find("sh") != -1:
        secid = "{}.{}".format(market_map["sh"], symbol.replace("sh", ""))
    elif symbol.find("csi") != -1:
        secid = "{}.{}".format(market_map["csi"], symbol.replace("csi", ""))
    else:
        return pd.DataFrame()
    params = {
        "cb": "jQuery1124033485574041163946_1596700547000",
        "secid": secid,
        "fields1": "f1,f2,f3,f4,f5",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
        "klt": "101",  # 日频率
        "fqt": "0",
        "beg": start_date,
        "end": end_date,
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{") : -2])
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    # check temp_df data availability before further transformations which may raise errors
    if temp_df.empty:
        return pd.DataFrame()
    temp_df.columns = ["date", "open", "close", "high", "low", "volume", "amount", "_"]
    temp_df = temp_df[["date", "open", "close", "high", "low", "volume", "amount"]]
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    temp_df["amount"] = pd.to_numeric(temp_df["amount"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_zh_index_daily_df = stock_zh_index_daily(symbol="sh000510")
    print(stock_zh_index_daily_df)

    stock_zh_index_spot_sina_df = stock_zh_index_spot_sina()
    print(stock_zh_index_spot_sina_df)

    stock_zh_index_spot_em_df = stock_zh_index_spot_em(symbol="沪深重要指数")
    print(stock_zh_index_spot_em_df)

    stock_zh_index_spot_em_df = stock_zh_index_spot_em(symbol="上证系列指数")
    print(stock_zh_index_spot_em_df)

    stock_zh_index_spot_em_df = stock_zh_index_spot_em(symbol="深证系列指数")
    print(stock_zh_index_spot_em_df)

    stock_zh_index_spot_em_df = stock_zh_index_spot_em(symbol="指数成份")
    print(stock_zh_index_spot_em_df)

    stock_zh_index_spot_em_df = stock_zh_index_spot_em(symbol="中证系列指数")
    print(stock_zh_index_spot_em_df)

    stock_zh_index_daily_tx_df = stock_zh_index_daily_tx(symbol="sh000919")
    print(stock_zh_index_daily_tx_df)

    stock_zh_index_daily_em_df = stock_zh_index_daily_em(symbol="sz399812")
    print(stock_zh_index_daily_em_df)



# !/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/4/1 19:20
Desc: 雪球-行情中心-个股
https://xueqiu.com/S/SH513520
"""

import re
from datetime import datetime

import pandas as pd
import requests


def _convert_timestamp(timestamp_ms: int) -> str:
    """
    时间戳转换为字符串时间
    :param timestamp_ms: 时间戳
    :type timestamp_ms: int
    :return: 字符串
    :rtype: str
    """
    timestamp_s = timestamp_ms / 1000
    datetime_obj = datetime.fromtimestamp(timestamp_s)
    return datetime_obj.strftime("%Y-%m-%d %H:%M:%S")


def stock_individual_spot_xq(
    symbol: str = "SH600000",
    token: str = None,
    timeout: float = None,
) -> pd.DataFrame:
    """
    雪球-行情中心-个股
    https://xueqiu.com/S/SH600000
    :param symbol: 证券代码，可以是 A 股代码，A 股场内基金代码，A 股指数，美股代码, 美股指数
    :type symbol: str
    :param token: set xueqiu token
    :type token: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 证券最新行情
    :rtype: pandas.DataFrame
    """
    session = requests.Session()
    xq_a_token = token or "8227a6f1f070ca10a573ea273e25da157b017b89"
    headers = {
        "cookie": f"xq_a_token={xq_a_token};",
        "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 16_6 like Mac OS X) AppleWebKit/605.1.15 "
        "(KHTML, like Gecko) Version/16.6 Mobile/15E148 Safari/604.1",
    }
    session.get(url="https://xueqiu.com", headers=headers)
    url = f"https://stock.xueqiu.com/v5/stock/quote.json?symbol={symbol}&extend=detail"
    r = session.get(url, headers=headers, timeout=timeout)
    column_name_map = {
        "acc_unit_nav": "累计净值",
        "amount": "成交额",
        "amplitude": "振幅",
        "avg_price": "均价",
        "chg": "涨跌",
        "currency": "货币",
        "current": "现价",
        "current_year_percent": "今年以来涨幅",
        "dividend": "股息(TTM)",
        "dividend_yield": "股息率(TTM)",
        "eps": "每股收益",
        "exchange": "交易所",
        "float_market_capital": "流通值",
        "float_shares": "流通股",
        "found_date": "成立日期",
        "goodwill_in_net_assets": "净资产中的商誉",
        "high": "最高",
        "high52w": "52周最高",
        "iopv": "参考净值",
        "issue_date": "发行日期",
        "last_close": "昨收",
        "limit_down": "跌停",
        "limit_up": "涨停",
        "lot_size": "最小交易单位",
        "low": "最低",
        "low52w": "52周最低",
        "market_capital": "资产净值/总市值",
        "name": "名称",
        "nav_date": "净值日期",
        "navps": "每股净资产",
        "open": "今开",
        "pb": "市净率",
        "pe_forecast": "市盈率(动)",
        "pe_lyr": "市盈率(静)",
        "pe_ttm": "市盈率(TTM)",
        "percent": "涨幅",
        "premium_rate": "溢价率",
        "psr": "市销率",
        "symbol": "代码",
        "total_shares": "基金份额/总股本",
        "turnover_rate": "周转率",
        "unit_nav": "单位净值",
        "volume": "成交量",
        "time": "时间",
    }
    json_data = r.json()
    temp_df = pd.json_normalize(json_data["data"]["quote"])
    temp_df.columns = [
        *map(
            lambda x: column_name_map[x] if x in column_name_map.keys() else x,
            temp_df.columns,
        )  # 由于传入的 symbol 可能是个股，可能是指数，也可能是基金，所以这里取列的最大公约数，没有数据的列内容为 None
    ]
    temp_df = temp_df[
        list(
            filter(
                lambda x: re.search(pattern="[\u4e00-\u9fa5]", string=x),
                temp_df.columns,
            )  # 过滤 temp_df，留下包含汉字的列
        )
    ]
    temp_df = temp_df.T.reset_index()
    temp_df.columns = ["item", "value"]
    temp_df.loc[temp_df["item"] == "时间", "value"] = temp_df.loc[
        temp_df["item"] == "时间", "value"
    ].apply(lambda x: _convert_timestamp(int(x)) if x and not pd.isna(x) else None)
    temp_df.loc[temp_df["item"] == "发行日期", "value"] = temp_df.loc[
        temp_df["item"] == "发行日期", "value"
    ].apply(lambda x: _convert_timestamp(int(x)) if x and not pd.isna(x) else None)
    return temp_df


if __name__ == "__main__":
    stock_individual_spot_xq_df = stock_individual_spot_xq(symbol="BJ430139")
    print(stock_individual_spot_xq_df)

    stock_individual_spot_xq_df = stock_individual_spot_xq(symbol="SH600000")
    print(stock_individual_spot_xq_df)

    stock_individual_spot_xq_df = stock_individual_spot_xq(symbol="SPY")
    print(stock_individual_spot_xq_df)

    stock_individual_spot_xq_df = stock_individual_spot_xq(symbol=".INX")
    print(stock_individual_spot_xq_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/4/7 17:36
Desc: http://kcb.sse.com.cn/renewal/#
"""

import requests
import pandas as pd

# TODO


def stock_kcb_renewal():
    url = "http://query.sse.com.cn/statusAction.do"
    params = {
        "isPagination": "true",
        "sqlId": "SH_XM_LB",
        "pageHelp.pageSize": "20",
        "offerType": "",
        "commitiResult": "",
        "registeResult": "",
        "province": "",
        "csrcCode": "",
        "currStatus": "",
        "order": "updateDate|desc,stockAuditNum|desc",
        "keyword": "",
        "auditApplyDateBegin": "",
        "auditApplyDateEnd": "",
        "pageHelp.pageNo": "1",
        "pageHelp.beginPage": "1",
        "pageHelp.endPage": "1",
        "_": "1649322742207",
    }
    headers = {
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "http://kcb.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.60 Safari/537.36",
    }

    for page in range(1, 37):
        print(page)
        params.update(
            {
                "pageHelp.pageNo": page,
                "pageHelp.beginPage": page,
                "pageHelp.endPage": page,
            }
        )
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"])
        # 处理下 temp_df 里面的字段就可以了
        print(temp_df)
        break



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/3/13 21:50
Desc: 东方财富网-数据中心-特色数据-期权风险分析
https://data.eastmoney.com/other/riskanal.html
"""

import pandas as pd

from akshare.utils.func import fetch_paginated_data


def option_risk_analysis_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-期权风险分析
    https://data.eastmoney.com/other/riskanal.html
    :return: 期权风险分析
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "fid": "f12",
        "po": "1",
        "pz": "100",
        "pn": "1",
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fields": "f1,f2,f3,f12,f13,f14,f302,f303,f325,f326,f327,f329,f328,f301,f152,f154",
        "fs": "m:10",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "-",
        "-",
        "最新价",
        "涨跌幅",
        "期权代码",
        "-",
        "期权名称",
        "-",
        "-",
        "到期日",
        "杠杆比率",
        "实际杠杆比率",
        "Delta",
        "Gamma",
        "Vega",
        "Theta",
        "Rho",
    ]
    temp_df = temp_df[
        [
            "期权代码",
            "期权名称",
            "最新价",
            "涨跌幅",
            "杠杆比率",
            "实际杠杆比率",
            "Delta",
            "Gamma",
            "Vega",
            "Rho",
            "Theta",
            "到期日",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["杠杆比率"] = pd.to_numeric(temp_df["杠杆比率"], errors="coerce")
    temp_df["实际杠杆比率"] = pd.to_numeric(temp_df["实际杠杆比率"], errors="coerce")
    temp_df["Delta"] = pd.to_numeric(temp_df["Delta"], errors="coerce")
    temp_df["Gamma"] = pd.to_numeric(temp_df["Gamma"], errors="coerce")
    temp_df["Vega"] = pd.to_numeric(temp_df["Vega"], errors="coerce")
    temp_df["Rho"] = pd.to_numeric(temp_df["Rho"], errors="coerce")
    temp_df["Theta"] = pd.to_numeric(temp_df["Theta"], errors="coerce")
    temp_df["到期日"] = pd.to_datetime(
        temp_df["到期日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    option_risk_analysis_em_df = option_risk_analysis_em()
    print(option_risk_analysis_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/10 19:00
Desc: 东方财富网-数据中心-资金流向
https://data.eastmoney.com/zjlx/detail.html
"""

import math
import time
from functools import lru_cache

import pandas as pd
import requests

from akshare.utils.func import fetch_paginated_data
from akshare.utils.tqdm import get_tqdm


def stock_individual_fund_flow(
    stock: str = "600094", market: str = "sh"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-个股
    https://data.eastmoney.com/zjlx/detail.html
    :param stock: 股票代码
    :type stock: str
    :param market: 股票市场; 上海证券交易所: sh, 深证证券交易所: sz, 北京证券交易所: bj;
    :type market: str
    :return: 近期个股的资金流数据
    :rtype: pandas.DataFrame
    """
    market_map = {"sh": 1, "sz": 0, "bj": 0}
    url = "https://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
    }
    params = {
        "lmt": "0",
        "klt": "101",
        "secid": f"{market_map[market]}.{stock}",
        "fields1": "f1,f2,f3,f7",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "_": int(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    content_list = data_json["data"]["klines"]
    temp_df = pd.DataFrame([item.split(",") for item in content_list])
    temp_df.columns = [
        "日期",
        "主力净流入-净额",
        "小单净流入-净额",
        "中单净流入-净额",
        "大单净流入-净额",
        "超大单净流入-净额",
        "主力净流入-净占比",
        "小单净流入-净占比",
        "中单净流入-净占比",
        "大单净流入-净占比",
        "超大单净流入-净占比",
        "收盘价",
        "涨跌幅",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "日期",
            "收盘价",
            "涨跌幅",
            "主力净流入-净额",
            "主力净流入-净占比",
            "超大单净流入-净额",
            "超大单净流入-净占比",
            "大单净流入-净额",
            "大单净流入-净占比",
            "中单净流入-净额",
            "中单净流入-净占比",
            "小单净流入-净额",
            "小单净流入-净占比",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["主力净流入-净额"] = pd.to_numeric(
        temp_df["主力净流入-净额"], errors="coerce"
    )
    temp_df["小单净流入-净额"] = pd.to_numeric(
        temp_df["小单净流入-净额"], errors="coerce"
    )
    temp_df["中单净流入-净额"] = pd.to_numeric(
        temp_df["中单净流入-净额"], errors="coerce"
    )
    temp_df["大单净流入-净额"] = pd.to_numeric(
        temp_df["大单净流入-净额"], errors="coerce"
    )
    temp_df["超大单净流入-净额"] = pd.to_numeric(
        temp_df["超大单净流入-净额"], errors="coerce"
    )
    temp_df["主力净流入-净占比"] = pd.to_numeric(
        temp_df["主力净流入-净占比"], errors="coerce"
    )
    temp_df["小单净流入-净占比"] = pd.to_numeric(
        temp_df["小单净流入-净占比"], errors="coerce"
    )
    temp_df["中单净流入-净占比"] = pd.to_numeric(
        temp_df["中单净流入-净占比"], errors="coerce"
    )
    temp_df["大单净流入-净占比"] = pd.to_numeric(
        temp_df["大单净流入-净占比"], errors="coerce"
    )
    temp_df["超大单净流入-净占比"] = pd.to_numeric(
        temp_df["超大单净流入-净占比"], errors="coerce"
    )
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    return temp_df


def stock_individual_fund_flow_rank(indicator: str = "5日") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-排名
    https://data.eastmoney.com/zjlx/detail.html
    :param indicator: choice of {"今日", "3日", "5日", "10日"}
    :type indicator: str
    :return: 指定 indicator 资金流向排行
    :rtype: pandas.DataFrame
    """
    indicator_map = {
        "今日": [
            "f62",
            "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124",
        ],
        "3日": [
            "f267",
            "f12,f14,f2,f127,f267,f268,f269,f270,f271,f272,f273,f274,f275,f276,f257,f258,f124",
        ],
        "5日": [
            "f164",
            "f12,f14,f2,f109,f164,f165,f166,f167,f168,f169,f170,f171,f172,f173,f257,f258,f124",
        ],
        "10日": [
            "f174",
            "f12,f14,f2,f160,f174,f175,f176,f177,f178,f179,f180,f181,f182,f183,f260,f261,f124",
        ],
    }
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "fid": indicator_map[indicator][0],
        "po": "1",
        "pz": "100",
        "pn": "1",
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fs": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2,m:0+t:7+f:!2,m:1+t:3+f:!2",
        "fields": indicator_map[indicator][1],
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = math.ceil(data_json["data"]["total"] / 100)
    temp_list = []
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pn": page,
            }
        )
        r = requests.get(url, params=params, timeout=15)
        data_json = r.json()
        inner_temp_df = pd.DataFrame(data_json["data"]["diff"])
        temp_list.append(inner_temp_df)
    temp_df = pd.concat(temp_list, ignore_index=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    if indicator == "今日":
        temp_df.columns = [
            "序号",
            "最新价",
            "今日涨跌幅",
            "代码",
            "名称",
            "今日主力净流入-净额",
            "今日超大单净流入-净额",
            "今日超大单净流入-净占比",
            "今日大单净流入-净额",
            "今日大单净流入-净占比",
            "今日中单净流入-净额",
            "今日中单净流入-净占比",
            "今日小单净流入-净额",
            "今日小单净流入-净占比",
            "_",
            "今日主力净流入-净占比",
            "_",
            "_",
            "_",
        ]
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "今日涨跌幅",
                "今日主力净流入-净额",
                "今日主力净流入-净占比",
                "今日超大单净流入-净额",
                "今日超大单净流入-净占比",
                "今日大单净流入-净额",
                "今日大单净流入-净占比",
                "今日中单净流入-净额",
                "今日中单净流入-净占比",
                "今日小单净流入-净额",
                "今日小单净流入-净占比",
            ]
        ]
    elif indicator == "3日":
        temp_df.columns = [
            "序号",
            "最新价",
            "代码",
            "名称",
            "_",
            "3日涨跌幅",
            "_",
            "_",
            "_",
            "3日主力净流入-净额",
            "3日主力净流入-净占比",
            "3日超大单净流入-净额",
            "3日超大单净流入-净占比",
            "3日大单净流入-净额",
            "3日大单净流入-净占比",
            "3日中单净流入-净额",
            "3日中单净流入-净占比",
            "3日小单净流入-净额",
            "3日小单净流入-净占比",
        ]
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "3日涨跌幅",
                "3日主力净流入-净额",
                "3日主力净流入-净占比",
                "3日超大单净流入-净额",
                "3日超大单净流入-净占比",
                "3日大单净流入-净额",
                "3日大单净流入-净占比",
                "3日中单净流入-净额",
                "3日中单净流入-净占比",
                "3日小单净流入-净额",
                "3日小单净流入-净占比",
            ]
        ]
    elif indicator == "5日":
        temp_df.columns = [
            "序号",
            "最新价",
            "代码",
            "名称",
            "5日涨跌幅",
            "_",
            "5日主力净流入-净额",
            "5日主力净流入-净占比",
            "5日超大单净流入-净额",
            "5日超大单净流入-净占比",
            "5日大单净流入-净额",
            "5日大单净流入-净占比",
            "5日中单净流入-净额",
            "5日中单净流入-净占比",
            "5日小单净流入-净额",
            "5日小单净流入-净占比",
            "_",
            "_",
            "_",
        ]
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "5日涨跌幅",
                "5日主力净流入-净额",
                "5日主力净流入-净占比",
                "5日超大单净流入-净额",
                "5日超大单净流入-净占比",
                "5日大单净流入-净额",
                "5日大单净流入-净占比",
                "5日中单净流入-净额",
                "5日中单净流入-净占比",
                "5日小单净流入-净额",
                "5日小单净流入-净占比",
            ]
        ]
    elif indicator == "10日":
        temp_df.columns = [
            "序号",
            "最新价",
            "代码",
            "名称",
            "_",
            "10日涨跌幅",
            "10日主力净流入-净额",
            "10日主力净流入-净占比",
            "10日超大单净流入-净额",
            "10日超大单净流入-净占比",
            "10日大单净流入-净额",
            "10日大单净流入-净占比",
            "10日中单净流入-净额",
            "10日中单净流入-净占比",
            "10日小单净流入-净额",
            "10日小单净流入-净占比",
            "_",
            "_",
            "_",
        ]
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "10日涨跌幅",
                "10日主力净流入-净额",
                "10日主力净流入-净占比",
                "10日超大单净流入-净额",
                "10日超大单净流入-净占比",
                "10日大单净流入-净额",
                "10日大单净流入-净占比",
                "10日中单净流入-净额",
                "10日中单净流入-净占比",
                "10日小单净流入-净额",
                "10日小单净流入-净占比",
            ]
        ]
    return temp_df


def stock_market_fund_flow() -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-大盘
    https://data.eastmoney.com/zjlx/dpzjlx.html
    :return: 近期大盘的资金流数据
    :rtype: pandas.DataFrame
    """
    url = "https://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
    }
    params = {
        "lmt": "0",
        "klt": "101",
        "secid": "1.000001",
        "secid2": "0.399001",
        "fields1": "f1,f2,f3,f7",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "_": int(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    content_list = data_json["data"]["klines"]
    temp_df = pd.DataFrame([item.split(",") for item in content_list])
    temp_df.columns = [
        "日期",
        "主力净流入-净额",
        "小单净流入-净额",
        "中单净流入-净额",
        "大单净流入-净额",
        "超大单净流入-净额",
        "主力净流入-净占比",
        "小单净流入-净占比",
        "中单净流入-净占比",
        "大单净流入-净占比",
        "超大单净流入-净占比",
        "上证-收盘价",
        "上证-涨跌幅",
        "深证-收盘价",
        "深证-涨跌幅",
    ]
    temp_df = temp_df[
        [
            "日期",
            "上证-收盘价",
            "上证-涨跌幅",
            "深证-收盘价",
            "深证-涨跌幅",
            "主力净流入-净额",
            "主力净流入-净占比",
            "超大单净流入-净额",
            "超大单净流入-净占比",
            "大单净流入-净额",
            "大单净流入-净占比",
            "中单净流入-净额",
            "中单净流入-净占比",
            "小单净流入-净额",
            "小单净流入-净占比",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["上证-收盘价"] = pd.to_numeric(temp_df["上证-收盘价"], errors="coerce")
    temp_df["上证-涨跌幅"] = pd.to_numeric(temp_df["上证-涨跌幅"], errors="coerce")
    temp_df["深证-收盘价"] = pd.to_numeric(temp_df["深证-收盘价"], errors="coerce")
    temp_df["深证-涨跌幅"] = pd.to_numeric(temp_df["深证-涨跌幅"], errors="coerce")
    temp_df["主力净流入-净额"] = pd.to_numeric(
        temp_df["主力净流入-净额"], errors="coerce"
    )
    temp_df["主力净流入-净占比"] = pd.to_numeric(
        temp_df["主力净流入-净占比"], errors="coerce"
    )
    temp_df["超大单净流入-净额"] = pd.to_numeric(
        temp_df["超大单净流入-净额"], errors="coerce"
    )
    temp_df["超大单净流入-净占比"] = pd.to_numeric(
        temp_df["超大单净流入-净占比"], errors="coerce"
    )
    temp_df["大单净流入-净额"] = pd.to_numeric(
        temp_df["大单净流入-净额"], errors="coerce"
    )
    temp_df["大单净流入-净占比"] = pd.to_numeric(
        temp_df["大单净流入-净占比"], errors="coerce"
    )
    temp_df["中单净流入-净额"] = pd.to_numeric(
        temp_df["中单净流入-净额"], errors="coerce"
    )
    temp_df["中单净流入-净占比"] = pd.to_numeric(
        temp_df["中单净流入-净占比"], errors="coerce"
    )
    temp_df["小单净流入-净额"] = pd.to_numeric(
        temp_df["小单净流入-净额"], errors="coerce"
    )
    temp_df["小单净流入-净占比"] = pd.to_numeric(
        temp_df["小单净流入-净占比"], errors="coerce"
    )
    return temp_df


def stock_sector_fund_flow_rank(
    indicator: str = "今日", sector_type: str = "行业资金流"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-板块资金流-排名
    https://data.eastmoney.com/bkzj/hy.html
    :param indicator: choice of {"今日", "5日", "10日"}
    :type indicator: str
    :param sector_type: choice of {"行业资金流", "概念资金流", "地域资金流"}
    :type sector_type: str
    :return: 指定参数的资金流排名数据
    :rtype: pandas.DataFrame
    """
    sector_type_map = {"行业资金流": "2", "概念资金流": "3", "地域资金流": "1"}
    indicator_map = {
        "今日": [
            "f62",
            "1",
            "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124",
        ],
        "5日": [
            "f164",
            "5",
            "f12,f14,f2,f109,f164,f165,f166,f167,f168,f169,f170,f171,f172,f173,f257,f258,f124",
        ],
        "10日": [
            "f174",
            "10",
            "f12,f14,f2,f160,f174,f175,f176,f177,f178,f179,f180,f181,f182,f183,f260,f261,f124",
        ],
    }
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
    }
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fltt": "2",
        "invt": "2",
        "fid0": indicator_map[indicator][0],
        "fs": f"m:90 t:{sector_type_map[sector_type]}",
        "stat": indicator_map[indicator][1],
        "fields": indicator_map[indicator][2],
        "rt": "52975239",
        "_": int(time.time() * 1000),
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_page = math.ceil(data_json["data"]["total"] / 100)
    temp_list = []
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pn": page,
            }
        )
        r = requests.get(url, params=params, timeout=15)
        data_json = r.json()
        inner_temp_df = pd.DataFrame(data_json["data"]["diff"])
        temp_list.append(inner_temp_df)
    temp_df = pd.concat(temp_list, ignore_index=True)

    if indicator == "今日":
        temp_df.columns = [
            "-",
            "今日涨跌幅",
            "_",
            "名称",
            "今日主力净流入-净额",
            "今日超大单净流入-净额",
            "今日超大单净流入-净占比",
            "今日大单净流入-净额",
            "今日大单净流入-净占比",
            "今日中单净流入-净额",
            "今日中单净流入-净占比",
            "今日小单净流入-净额",
            "今日小单净流入-净占比",
            "-",
            "今日主力净流入-净占比",
            "今日主力净流入最大股",
            "今日主力净流入最大股代码",
            "是否净流入",
        ]

        temp_df = temp_df[
            [
                "名称",
                "今日涨跌幅",
                "今日主力净流入-净额",
                "今日主力净流入-净占比",
                "今日超大单净流入-净额",
                "今日超大单净流入-净占比",
                "今日大单净流入-净额",
                "今日大单净流入-净占比",
                "今日中单净流入-净额",
                "今日中单净流入-净占比",
                "今日小单净流入-净额",
                "今日小单净流入-净占比",
                "今日主力净流入最大股",
            ]
        ]
        temp_df["今日主力净流入-净额"] = pd.to_numeric(
            temp_df["今日主力净流入-净额"], errors="coerce"
        )
        temp_df.sort_values(["今日主力净流入-净额"], ascending=False, inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["index"] = range(1, len(temp_df) + 1)
        temp_df.rename({"index": "序号"}, axis=1, inplace=True)
    elif indicator == "5日":
        temp_df.columns = [
            "-",
            "_",
            "名称",
            "5日涨跌幅",
            "_",
            "5日主力净流入-净额",
            "5日主力净流入-净占比",
            "5日超大单净流入-净额",
            "5日超大单净流入-净占比",
            "5日大单净流入-净额",
            "5日大单净流入-净占比",
            "5日中单净流入-净额",
            "5日中单净流入-净占比",
            "5日小单净流入-净额",
            "5日小单净流入-净占比",
            "5日主力净流入最大股",
            "_",
            "_",
        ]

        temp_df = temp_df[
            [
                "名称",
                "5日涨跌幅",
                "5日主力净流入-净额",
                "5日主力净流入-净占比",
                "5日超大单净流入-净额",
                "5日超大单净流入-净占比",
                "5日大单净流入-净额",
                "5日大单净流入-净占比",
                "5日中单净流入-净额",
                "5日中单净流入-净占比",
                "5日小单净流入-净额",
                "5日小单净流入-净占比",
                "5日主力净流入最大股",
            ]
        ]
        temp_df.sort_values(["5日主力净流入-净额"], ascending=False, inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["index"] = range(1, len(temp_df) + 1)
        temp_df.rename({"index": "序号"}, axis=1, inplace=True)
    elif indicator == "10日":
        temp_df.columns = [
            "-",
            "_",
            "名称",
            "_",
            "10日涨跌幅",
            "10日主力净流入-净额",
            "10日主力净流入-净占比",
            "10日超大单净流入-净额",
            "10日超大单净流入-净占比",
            "10日大单净流入-净额",
            "10日大单净流入-净占比",
            "10日中单净流入-净额",
            "10日中单净流入-净占比",
            "10日小单净流入-净额",
            "10日小单净流入-净占比",
            "10日主力净流入最大股",
            "_",
            "_",
        ]

        temp_df = temp_df[
            [
                "名称",
                "10日涨跌幅",
                "10日主力净流入-净额",
                "10日主力净流入-净占比",
                "10日超大单净流入-净额",
                "10日超大单净流入-净占比",
                "10日大单净流入-净额",
                "10日大单净流入-净占比",
                "10日中单净流入-净额",
                "10日中单净流入-净占比",
                "10日小单净流入-净额",
                "10日小单净流入-净占比",
                "10日主力净流入最大股",
            ]
        ]
        temp_df.sort_values(["10日主力净流入-净额"], ascending=False, inplace=True)
        temp_df.reset_index(inplace=True)
        temp_df["index"] = range(1, len(temp_df) + 1)
        temp_df.rename({"index": "序号"}, axis=1, inplace=True)
    return temp_df


@lru_cache()
def _get_stock_sector_fund_flow_summary_code() -> dict:
    """
    东方财富网-数据中心-资金流向-行业板块
    https://data.eastmoney.com/bkzj/gn.html
    :return: 行业板块与代码字典
    :rtype: dict
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fltt": "2",
        "invt": "2",
        "fid0": "f62",
        "fs": "m:90 t:2",
        "stat": "1",
        "fields": "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124",
        "rt": "52975239",
        "_": int(time.time() * 1000),
    }
    temp_df = fetch_paginated_data(url, params)
    name_code_map = dict(zip(temp_df["f14"], temp_df["f12"]))
    return name_code_map


def stock_sector_fund_flow_summary(
    symbol: str = "电源设备", indicator: str = "今日"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-行业资金流-xx行业个股资金流
    https://data.eastmoney.com/bkzj/BK1034.html
    :param symbol: 行业名称
    :type symbol: str
    :param indicator: choice of {"今日", "5日", "10日"}
    :type indicator: str
    :return: xx行业个股资金流
    :rtype: pandas.DataFrame
    """
    code_name_map = _get_stock_sector_fund_flow_summary_code()
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    if indicator == "今日":
        params = {
            "fid": "f62",
            "po": "1",
            "pz": "5000",
            "pn": "1",
            "np": "2",
            "fltt": "2",
            "invt": "2",
            "fs": f"b:{code_name_map[symbol]}",
            "fields": "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124,f1,f13",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"]["diff"]).T
        temp_df.reset_index(inplace=True)
        temp_df["index"] = temp_df["index"].astype(int) + 1
        temp_df.rename(
            columns={
                "index": "序号",
                "f12": "代码",
                "f14": "名称",
                "f2": "最新价",
                "f3": "今天涨跌幅",
                "f62": "今日主力净流入-净额",
                "f184": "今日主力净流入-净占比",
                "f66": "今日超大单净流入-净额",
                "f69": "今日超大单净流入-净占比",
                "f72": "今日大单净流入-净额",
                "f75": "今日大单净流入-净占比",
                "f78": "今日中单净流入-净额",
                "f81": "今日中单净流入-净占比",
                "f84": "今日小单净流入-净额",
                "f87": "今日小单净流入-净占比",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "今天涨跌幅",
                "今日主力净流入-净额",
                "今日主力净流入-净占比",
                "今日超大单净流入-净额",
                "今日超大单净流入-净占比",
                "今日大单净流入-净额",
                "今日大单净流入-净占比",
                "今日中单净流入-净额",
                "今日中单净流入-净占比",
                "今日小单净流入-净额",
                "今日小单净流入-净占比",
            ]
        ]
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["今天涨跌幅"] = pd.to_numeric(temp_df["今天涨跌幅"], errors="coerce")
        temp_df["今日主力净流入-净额"] = pd.to_numeric(
            temp_df["今日主力净流入-净额"], errors="coerce"
        )
        temp_df["今日主力净流入-净占比"] = pd.to_numeric(
            temp_df["今日主力净流入-净占比"], errors="coerce"
        )
        temp_df["今日超大单净流入-净额"] = pd.to_numeric(
            temp_df["今日超大单净流入-净额"], errors="coerce"
        )
        temp_df["今日超大单净流入-净占比"] = pd.to_numeric(
            temp_df["今日超大单净流入-净占比"], errors="coerce"
        )
        temp_df["今日大单净流入-净额"] = pd.to_numeric(
            temp_df["今日大单净流入-净额"], errors="coerce"
        )
        temp_df["今日大单净流入-净占比"] = pd.to_numeric(
            temp_df["今日大单净流入-净占比"], errors="coerce"
        )
        temp_df["今日中单净流入-净额"] = pd.to_numeric(
            temp_df["今日中单净流入-净额"], errors="coerce"
        )
        temp_df["今日中单净流入-净占比"] = pd.to_numeric(
            temp_df["今日中单净流入-净占比"], errors="coerce"
        )
        temp_df["今日小单净流入-净额"] = pd.to_numeric(
            temp_df["今日小单净流入-净额"], errors="coerce"
        )
        temp_df["今日小单净流入-净占比"] = pd.to_numeric(
            temp_df["今日小单净流入-净占比"], errors="coerce"
        )
        return temp_df
    if indicator == "5日":
        params = {
            "fid": "f164",
            "po": "1",
            "pz": "50000",
            "pn": "1",
            "np": "2",
            "fltt": "2",
            "invt": "2",
            "fs": f"b:{code_name_map[symbol]}",
            "fields": "f12,f14,f2,f109,f164,f165,f166,f167,f168,f169,f170,f171,f172,f173,f257,f258,f124,f1,f13",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"]["diff"]).T
        temp_df.reset_index(inplace=True)
        temp_df["index"] = temp_df["index"].astype(int) + 1
        temp_df.rename(
            columns={
                "index": "序号",
                "f12": "代码",
                "f14": "名称",
                "f2": "最新价",
                "f109": "5日涨跌幅",
                "f164": "5日主力净流入-净额",
                "f165": "5日主力净流入-净占比",
                "f166": "5日超大单净流入-净额",
                "f167": "5日超大单净流入-净占比",
                "f168": "5日大单净流入-净额",
                "f169": "5日大单净流入-净占比",
                "f170": "5日中单净流入-净额",
                "f171": "5日中单净流入-净占比",
                "f172": "5日小单净流入-净额",
                "f173": "5日小单净流入-净占比",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "5日涨跌幅",
                "5日主力净流入-净额",
                "5日主力净流入-净占比",
                "5日超大单净流入-净额",
                "5日超大单净流入-净占比",
                "5日大单净流入-净额",
                "5日大单净流入-净占比",
                "5日中单净流入-净额",
                "5日中单净流入-净占比",
                "5日小单净流入-净额",
                "5日小单净流入-净占比",
            ]
        ]
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["5日涨跌幅"] = pd.to_numeric(temp_df["5日涨跌幅"], errors="coerce")
        temp_df["5日主力净流入-净额"] = pd.to_numeric(
            temp_df["5日主力净流入-净额"], errors="coerce"
        )
        temp_df["5日主力净流入-净占比"] = pd.to_numeric(
            temp_df["5日主力净流入-净占比"], errors="coerce"
        )
        temp_df["5日超大单净流入-净额"] = pd.to_numeric(
            temp_df["5日超大单净流入-净额"], errors="coerce"
        )
        temp_df["5日超大单净流入-净占比"] = pd.to_numeric(
            temp_df["5日超大单净流入-净占比"], errors="coerce"
        )
        temp_df["5日大单净流入-净额"] = pd.to_numeric(
            temp_df["5日大单净流入-净额"], errors="coerce"
        )
        temp_df["5日大单净流入-净占比"] = pd.to_numeric(
            temp_df["5日大单净流入-净占比"], errors="coerce"
        )
        temp_df["5日中单净流入-净额"] = pd.to_numeric(
            temp_df["5日中单净流入-净额"], errors="coerce"
        )
        temp_df["5日中单净流入-净占比"] = pd.to_numeric(
            temp_df["5日中单净流入-净占比"], errors="coerce"
        )
        temp_df["5日小单净流入-净额"] = pd.to_numeric(
            temp_df["5日小单净流入-净额"], errors="coerce"
        )
        temp_df["5日小单净流入-净占比"] = pd.to_numeric(
            temp_df["5日小单净流入-净占比"], errors="coerce"
        )
        return temp_df
    if indicator == "10日":
        params = {
            "fid": "f174",
            "po": "1",
            "pz": "50000",
            "pn": "1",
            "np": "2",
            "fltt": "2",
            "invt": "2",
            "fs": f"b:{code_name_map[symbol]}",
            "fields": "f12,f14,f2,f160,f174,f175,f176,f177,f178,f179,f180,f181,f182,f183,f260,f261,f124,f1,f13",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"]["diff"]).T
        temp_df.reset_index(inplace=True)
        temp_df["index"] = temp_df["index"].astype(int) + 1
        temp_df.rename(
            columns={
                "index": "序号",
                "f12": "代码",
                "f14": "名称",
                "f2": "最新价",
                "f160": "10日涨跌幅",
                "f174": "10日主力净流入-净额",
                "f175": "10日主力净流入-净占比",
                "f176": "10日超大单净流入-净额",
                "f177": "10日超大单净流入-净占比",
                "f178": "10日大单净流入-净额",
                "f179": "10日大单净流入-净占比",
                "f180": "10日中单净流入-净额",
                "f181": "10日中单净流入-净占比",
                "f182": "10日小单净流入-净额",
                "f183": "10日小单净流入-净占比",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "序号",
                "代码",
                "名称",
                "最新价",
                "10日涨跌幅",
                "10日主力净流入-净额",
                "10日主力净流入-净占比",
                "10日超大单净流入-净额",
                "10日超大单净流入-净占比",
                "10日大单净流入-净额",
                "10日大单净流入-净占比",
                "10日中单净流入-净额",
                "10日中单净流入-净占比",
                "10日小单净流入-净额",
                "10日小单净流入-净占比",
            ]
        ]
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["10日涨跌幅"] = pd.to_numeric(temp_df["10日涨跌幅"], errors="coerce")
        temp_df["10日主力净流入-净额"] = pd.to_numeric(
            temp_df["10日主力净流入-净额"], errors="coerce"
        )
        temp_df["10日主力净流入-净占比"] = pd.to_numeric(
            temp_df["10日主力净流入-净占比"], errors="coerce"
        )
        temp_df["10日超大单净流入-净额"] = pd.to_numeric(
            temp_df["10日超大单净流入-净额"], errors="coerce"
        )
        temp_df["10日超大单净流入-净占比"] = pd.to_numeric(
            temp_df["10日超大单净流入-净占比"], errors="coerce"
        )
        temp_df["10日大单净流入-净额"] = pd.to_numeric(
            temp_df["10日大单净流入-净额"], errors="coerce"
        )
        temp_df["10日大单净流入-净占比"] = pd.to_numeric(
            temp_df["10日大单净流入-净占比"], errors="coerce"
        )
        temp_df["10日中单净流入-净额"] = pd.to_numeric(
            temp_df["10日中单净流入-净额"], errors="coerce"
        )
        temp_df["10日中单净流入-净占比"] = pd.to_numeric(
            temp_df["10日中单净流入-净占比"], errors="coerce"
        )
        temp_df["10日小单净流入-净额"] = pd.to_numeric(
            temp_df["10日小单净流入-净额"], errors="coerce"
        )
        temp_df["10日小单净流入-净占比"] = pd.to_numeric(
            temp_df["10日小单净流入-净占比"], errors="coerce"
        )
        return temp_df
    else:
        return pd.DataFrame()


def stock_sector_fund_flow_hist(symbol: str = "汽车服务") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-行业资金流-行业历史资金流
    https://data.eastmoney.com/bkzj/BK1034.html
    :param symbol: 行业名称
    :type symbol: str
    :return: xx行业个股资金流
    :rtype: pandas.DataFrame
    """
    code_name_map = _get_stock_sector_fund_flow_summary_code()
    url = "https://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get"
    params = {
        "lmt": "0",
        "klt": "101",
        "fields1": "f1,f2,f3,f7",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
        "secid": f"90.{code_name_map[symbol]}",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "日期",
        "主力净流入-净额",
        "小单净流入-净额",
        "中单净流入-净额",
        "大单净流入-净额",
        "超大单净流入-净额",
        "主力净流入-净占比",
        "小单净流入-净占比",
        "中单净流入-净占比",
        "大单净流入-净占比",
        "超大单净流入-净占比",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "日期",
            "主力净流入-净额",
            "主力净流入-净占比",
            "超大单净流入-净额",
            "超大单净流入-净占比",
            "大单净流入-净额",
            "大单净流入-净占比",
            "中单净流入-净额",
            "中单净流入-净占比",
            "小单净流入-净额",
            "小单净流入-净占比",
        ]
    ]
    temp_df["主力净流入-净额"] = pd.to_numeric(
        temp_df["主力净流入-净额"], errors="coerce"
    )
    temp_df["主力净流入-净占比"] = pd.to_numeric(
        temp_df["主力净流入-净占比"], errors="coerce"
    )
    temp_df["超大单净流入-净额"] = pd.to_numeric(
        temp_df["超大单净流入-净额"], errors="coerce"
    )
    temp_df["超大单净流入-净占比"] = pd.to_numeric(
        temp_df["超大单净流入-净占比"], errors="coerce"
    )
    temp_df["大单净流入-净额"] = pd.to_numeric(
        temp_df["大单净流入-净额"], errors="coerce"
    )
    temp_df["大单净流入-净占比"] = pd.to_numeric(
        temp_df["大单净流入-净占比"], errors="coerce"
    )
    temp_df["中单净流入-净额"] = pd.to_numeric(
        temp_df["中单净流入-净额"], errors="coerce"
    )
    temp_df["中单净流入-净占比"] = pd.to_numeric(
        temp_df["中单净流入-净占比"], errors="coerce"
    )
    temp_df["小单净流入-净额"] = pd.to_numeric(
        temp_df["小单净流入-净额"], errors="coerce"
    )
    temp_df["小单净流入-净占比"] = pd.to_numeric(
        temp_df["小单净流入-净占比"], errors="coerce"
    )
    temp_df["日期"] = pd.to_datetime(temp_df["日期"]).dt.date
    return temp_df


@lru_cache()
def _get_stock_concept_fund_flow_summary_code() -> dict:
    """
    东方财富网-数据中心-资金流向-概念资金流
    https://data.eastmoney.com/bkzj/gn.html
    :return: 概念与代码字典
    :rtype: dict
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "fields": "f3,f12,f13,f14,f62",
        "fid": "f62",
        "fs": "m:90+t:3",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "_": int(time.time() * 1000),
    }
    temp_df = fetch_paginated_data(url, params)
    name_code_map = dict(zip(temp_df["f14"], temp_df["f12"]))
    return name_code_map


def stock_concept_fund_flow_hist(symbol: str = "数据要素") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-概念资金流-概念历史资金流
    https://data.eastmoney.com/bkzj/BK0574.html
    :param symbol: 概念名称
    :type symbol: str
    :return: 概念历史资金流
    :rtype: pandas.DataFrame
    """
    code_name_map = _get_stock_concept_fund_flow_summary_code()
    url = "https://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get"
    params = {
        "lmt": "0",
        "klt": "101",
        "fields1": "f1,f2,f3,f7",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65",
        "secid": f"90.{code_name_map[symbol]}",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "日期",
        "主力净流入-净额",
        "小单净流入-净额",
        "中单净流入-净额",
        "大单净流入-净额",
        "超大单净流入-净额",
        "主力净流入-净占比",
        "小单净流入-净占比",
        "中单净流入-净占比",
        "大单净流入-净占比",
        "超大单净流入-净占比",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "日期",
            "主力净流入-净额",
            "主力净流入-净占比",
            "超大单净流入-净额",
            "超大单净流入-净占比",
            "大单净流入-净额",
            "大单净流入-净占比",
            "中单净流入-净额",
            "中单净流入-净占比",
            "小单净流入-净额",
            "小单净流入-净占比",
        ]
    ]
    temp_df["主力净流入-净额"] = pd.to_numeric(
        temp_df["主力净流入-净额"], errors="coerce"
    )
    temp_df["主力净流入-净占比"] = pd.to_numeric(
        temp_df["主力净流入-净占比"], errors="coerce"
    )
    temp_df["超大单净流入-净额"] = pd.to_numeric(
        temp_df["超大单净流入-净额"], errors="coerce"
    )
    temp_df["超大单净流入-净占比"] = pd.to_numeric(
        temp_df["超大单净流入-净占比"], errors="coerce"
    )
    temp_df["大单净流入-净额"] = pd.to_numeric(
        temp_df["大单净流入-净额"], errors="coerce"
    )
    temp_df["大单净流入-净占比"] = pd.to_numeric(
        temp_df["大单净流入-净占比"], errors="coerce"
    )
    temp_df["中单净流入-净额"] = pd.to_numeric(
        temp_df["中单净流入-净额"], errors="coerce"
    )
    temp_df["中单净流入-净占比"] = pd.to_numeric(
        temp_df["中单净流入-净占比"], errors="coerce"
    )
    temp_df["小单净流入-净额"] = pd.to_numeric(
        temp_df["小单净流入-净额"], errors="coerce"
    )
    temp_df["小单净流入-净占比"] = pd.to_numeric(
        temp_df["小单净流入-净占比"], errors="coerce"
    )
    temp_df["日期"] = pd.to_datetime(temp_df["日期"]).dt.date
    return temp_df


def stock_main_fund_flow(symbol: str = "全部股票") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-主力净流入排名
    https://data.eastmoney.com/zjlx/list.html
    :param symbol: 全部股票; choice of {"全部股票", "沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "沪市B股", "深市B股"}
    :type symbol: str
    :return: 主力净流入排名
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部股票": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2,m:0+t:7+f:!2,m:1+t:3+f:!2",
        "沪深A股": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2",
        "沪市A股": "m:1+t:2+f:!2,m:1+t:23+f:!2",
        "科创板": "m:1+t:23+f:!2",
        "深市A股": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2",
        "创业板": "m:0+t:80+f:!2",
        "沪市B股": "m:1+t:3+f:!2",
        "深市B股": "m:0+t:7+f:!2",
    }
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "fid": "f184",
        "po": "1",
        "pz": "100",
        "pn": "1",
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "fields": "f2,f3,f12,f13,f14,f62,f184,f225,f165,f263,f109,f175,f264,f160,f100,f124,f265,f1",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fs": symbol_map[symbol],
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.rename(
        columns={
            "index": "序号",
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f184": "今日排行榜-主力净占比",
            "f225": "今日排行榜-今日排名",
            "f3": "今日排行榜-今日涨跌",
            "f165": "5日排行榜-主力净占比",
            "f263": "5日排行榜-5日排名",
            "f109": "5日排行榜-5日涨跌",
            "f175": "10日排行榜-主力净占比",
            "f264": "10日排行榜-10日排名",
            "f160": "10日排行榜-10日涨跌",
            "f100": "所属板块",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "今日排行榜-主力净占比",
            "今日排行榜-今日排名",
            "今日排行榜-今日涨跌",
            "5日排行榜-主力净占比",
            "5日排行榜-5日排名",
            "5日排行榜-5日涨跌",
            "10日排行榜-主力净占比",
            "10日排行榜-10日排名",
            "10日排行榜-10日涨跌",
            "所属板块",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["今日排行榜-主力净占比"] = pd.to_numeric(
        temp_df["今日排行榜-主力净占比"], errors="coerce"
    )
    temp_df["今日排行榜-今日排名"] = pd.to_numeric(
        temp_df["今日排行榜-今日排名"], errors="coerce"
    )
    temp_df["今日排行榜-今日涨跌"] = pd.to_numeric(
        temp_df["今日排行榜-今日涨跌"], errors="coerce"
    )
    temp_df["5日排行榜-主力净占比"] = pd.to_numeric(
        temp_df["5日排行榜-主力净占比"], errors="coerce"
    )
    temp_df["5日排行榜-5日排名"] = pd.to_numeric(
        temp_df["5日排行榜-5日排名"], errors="coerce"
    )
    temp_df["5日排行榜-5日涨跌"] = pd.to_numeric(
        temp_df["5日排行榜-5日涨跌"], errors="coerce"
    )
    temp_df["10日排行榜-主力净占比"] = pd.to_numeric(
        temp_df["10日排行榜-主力净占比"], errors="coerce"
    )
    temp_df["10日排行榜-10日排名"] = pd.to_numeric(
        temp_df["10日排行榜-10日排名"], errors="coerce"
    )
    temp_df["10日排行榜-10日涨跌"] = pd.to_numeric(
        temp_df["10日排行榜-10日涨跌"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    stock_individual_fund_flow_df = stock_individual_fund_flow(
        stock="600094", market="sh"
    )
    print(stock_individual_fund_flow_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="今日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="3日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="5日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="10日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_market_fund_flow_df = stock_market_fund_flow()
    print(stock_market_fund_flow_df)

    stock_sector_fund_flow_rank_df = stock_sector_fund_flow_rank(
        indicator="今日", sector_type="地域资金流"
    )
    print(stock_sector_fund_flow_rank_df)

    stock_sector_fund_flow_rank_df = stock_sector_fund_flow_rank(
        indicator="今日", sector_type="行业资金流"
    )
    print(stock_sector_fund_flow_rank_df)

    stock_sector_fund_flow_rank_df = stock_sector_fund_flow_rank(
        indicator="今日", sector_type="概念资金流"
    )
    print(stock_sector_fund_flow_rank_df)

    stock_sector_fund_flow_summary_df = stock_sector_fund_flow_summary(
        symbol="文化传媒", indicator="今日"
    )
    print(stock_sector_fund_flow_summary_df)

    stock_sector_fund_flow_hist_df = stock_sector_fund_flow_hist(symbol="汽车服务")
    print(stock_sector_fund_flow_hist_df)

    stock_concept_fund_flow_hist_df = stock_concept_fund_flow_hist(symbol="半导体概念")
    print(stock_concept_fund_flow_hist_df)

    stock_main_fund_flow_df = stock_main_fund_flow(symbol="全部股票")
    print(stock_main_fund_flow_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/9/3 16:30
Desc: 主营构成
https://emweb.securities.eastmoney.com/PC_HSF10/BusinessAnalysis/Index?type=web&code=SH688041#
https://f10.emoney.cn/f10/zbyz/1000001
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_zygc_ym(symbol: str = "000001") -> pd.DataFrame:
    """
    益盟-F10-主营构成
    https://f10.emoney.cn/f10/zbyz/1000001
    :param symbol: 股票代码
    :type symbol: str
    :return: 主营构成
    :rtype: pandas.DataFrame
    """
    url = f"http://f10.emoney.cn/f10/zygc/{symbol}"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    year_list = [
        item.text.strip()
        for item in soup.find(attrs={"class": "swlab_t"}).find_all("li")
    ]
    big_df = pd.DataFrame()
    for i, item in enumerate(year_list, 2):
        temp_df = pd.read_html(StringIO(r.text), header=0)[i]
        temp_df.columns = [
            "分类方向",
            "分类",
            "营业收入",
            "营业收入-同比增长",
            "营业收入-占主营收入比",
            "营业成本",
            "营业成本-同比增长",
            "营业成本-占主营成本比",
            "毛利率",
            "毛利率-同比增长",
        ]
        temp_df["报告期"] = item
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df = big_df[
        [
            "报告期",
            "分类方向",
            "分类",
            "营业收入",
            "营业收入-同比增长",
            "营业收入-占主营收入比",
            "营业成本",
            "营业成本-同比增长",
            "营业成本-占主营成本比",
            "毛利率",
            "毛利率-同比增长",
        ]
    ]
    return big_df


def stock_zygc_em(symbol: str = "SH688041") -> pd.DataFrame:
    """
    东方财富网-个股-主营构成
    https://emweb.securities.eastmoney.com/PC_HSF10/BusinessAnalysis/Index?type=web&code=SH688041#
    :param symbol: 带市场标识的股票代码
    :type symbol: str
    :return: 主营构成
    :rtype: pandas.DataFrame
    """
    url = "https://emweb.securities.eastmoney.com/PC_HSF10/BusinessAnalysis/PageAjax"
    params = {"code": symbol}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["zygcfx"])
    temp_df.rename(
        columns={
            "SECUCODE": "-",
            "SECURITY_CODE": "股票代码",
            "REPORT_DATE": "报告日期",
            "MAINOP_TYPE": "分类类型",
            "ITEM_NAME": "主营构成",
            "MAIN_BUSINESS_INCOME": "主营收入",
            "MBI_RATIO": "收入比例",
            "MAIN_BUSINESS_COST": "主营成本",
            "MBC_RATIO": "成本比例",
            "MAIN_BUSINESS_RPOFIT": "主营利润",
            "MBR_RATIO": "利润比例",
            "GROSS_RPOFIT_RATIO": "毛利率",
            "RANK": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "股票代码",
            "报告日期",
            "分类类型",
            "主营构成",
            "主营收入",
            "收入比例",
            "主营成本",
            "成本比例",
            "主营利润",
            "利润比例",
            "毛利率",
        ]
    ]
    temp_df["报告日期"] = pd.to_datetime(temp_df["报告日期"], errors="coerce").dt.date
    temp_df["分类类型"] = temp_df["分类类型"].map(
        {"2": "按产品分类", "3": "按地区分类"}
    )
    temp_df["主营收入"] = pd.to_numeric(temp_df["主营收入"], errors="coerce")
    temp_df["收入比例"] = pd.to_numeric(temp_df["收入比例"], errors="coerce")
    temp_df["主营成本"] = pd.to_numeric(temp_df["主营成本"], errors="coerce")
    temp_df["成本比例"] = pd.to_numeric(temp_df["成本比例"], errors="coerce")
    temp_df["主营利润"] = pd.to_numeric(temp_df["主营利润"], errors="coerce")
    temp_df["利润比例"] = pd.to_numeric(temp_df["利润比例"], errors="coerce")
    temp_df["毛利率"] = pd.to_numeric(temp_df["毛利率"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_zygc_ym_df = stock_zygc_ym(symbol="000338")
    print(stock_zygc_ym_df)

    stock_zygc_em_df = stock_zygc_em(symbol="SH688041")
    print(stock_zygc_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/4/5 22:05
Desc: 乐估乐股-底部研究-巴菲特指标
https://legulegu.com/stockdata/marketcap-gdp
"""

import pandas as pd
import requests

from akshare.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_buffett_index_lg() -> pd.DataFrame:
    """
    乐估乐股-底部研究-巴菲特指标
    https://legulegu.com/stockdata/marketcap-gdp
    :return: 巴菲特指标
    :rtype: pandas.DataFrame
    """
    token = get_token_lg()
    url = "https://legulegu.com/api/stockdata/marketcap-gdp/get-marketcap-gdp"
    params = {"token": token}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/marketcap-gdp"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.rename(
        columns={
            "marketCap": "总市值",
            "gdp": "GDP",
            "close": "收盘价",
            "date": "日期",
            "quantileInAllHistory": "总历史分位数",
            "quantileInRecent10Years": "近十年分位数",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "日期",
            "收盘价",
            "总市值",
            "GDP",
            "近十年分位数",
            "总历史分位数",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], unit="ms").dt.date
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["GDP"] = pd.to_numeric(temp_df["GDP"], errors="coerce")
    temp_df["近十年分位数"] = pd.to_numeric(temp_df["近十年分位数"], errors="coerce")
    temp_df["总历史分位数"] = pd.to_numeric(temp_df["总历史分位数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_buffett_index_lg_df = stock_buffett_index_lg()
    print(stock_buffett_index_lg_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/9/3 21:00
Desc: 益盟-F10-管理层讨论与分析
https://f10.emoney.cn/f10/zbyz/1000001
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_mda_ym(symbol: str = "000001") -> pd.DataFrame:
    """
    益盟-F10-管理层讨论与分析
    https://f10.emoney.cn/f10/zbyz/1000001
    :param symbol: 股票代码
    :type symbol: str
    :return: 管理层讨论与分析
    :rtype: pandas.DataFrame
    """
    url = f"http://f10.emoney.cn/f10/zygc/{symbol}"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    year_list = [
        item.text.strip()
        for item in soup.find(attrs={"class": "swlab_t"}).find_all("li")
    ]
    talk_list = [
        item.text.strip().replace("\xa0", " ")
        for item in soup.find_all(attrs={"class": "cnt"})
    ]
    big_df = pd.DataFrame([year_list, talk_list]).T
    big_df.columns = ["报告期", "内容"]
    big_df.sort_values(by=["报告期"], ignore_index=True, inplace=True)
    return big_df


if __name__ == "__main__":
    stock_mda_ym_df = stock_mda_ym(symbol="000001")
    print(stock_mda_ym_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/11/5 16:00
Desc: 雪球-行情中心-沪深股市-内部交易
https://xueqiu.com/hq/insider
"""

import pandas as pd
import requests


def stock_inner_trade_xq() -> pd.DataFrame:
    """
    雪球-行情中心-沪深股市-内部交易
    https://xueqiu.com/hq/insider
    :return: 内部交易
    :rtype: pandas.DataFrame
    """
    url = "https://xueqiu.com/service/v5/stock/f10/cn/skholderchg"
    params = {
        "size": "100000",
        "page": "1",
        "extend": "true",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "xueqiu.com",
        "Pragma": "no-cache",
        "Referer": "https://xueqiu.com/hq",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="100", "Google Chrome";v="100"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/100.0.4896.127 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["items"])
    temp_df.columns = [
        "股票代码",
        "股票名称",
        "变动人",
        "-",
        "变动日期",
        "变动股数",
        "成交均价",
        "变动后持股数",
        "与董监高关系",
        "董监高职务",
    ]
    temp_df = temp_df[
        [
            "股票代码",
            "股票名称",
            "变动日期",
            "变动人",
            "变动股数",
            "成交均价",
            "变动后持股数",
            "与董监高关系",
            "董监高职务",
        ]
    ]
    temp_df["变动日期"] = (
        pd.to_datetime(temp_df["变动日期"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df["变动股数"] = pd.to_numeric(temp_df["变动股数"], errors="coerce")
    temp_df["成交均价"] = pd.to_numeric(temp_df["成交均价"], errors="coerce")
    temp_df["变动后持股数"] = pd.to_numeric(temp_df["变动后持股数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_inner_trade_xq_df = stock_inner_trade_xq()
    print(stock_inner_trade_xq_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/13 17:45
Desc: 上海期货交易所-交易所服务-业务数据-交易参数汇总查询
https://tsite.shfe.com.cn/bourseService/businessdata/summaryinquiry/
"""

import pandas as pd
import requests


def futures_contract_info_shfe(date: str = "20240513") -> pd.DataFrame:
    """
    上海期货交易所-交易所服务-业务数据-交易参数汇总查询
    https://tsite.shfe.com.cn/bourseService/businessdata/summaryinquiry/
    :param date: 查询日期; 交易日
    :type date: str
    :return: 交易参数汇总查询
    :rtype: pandas.DataFrame
    """
    url = f"https://tsite.shfe.com.cn/data/instrument/ContractBaseInfo{date}.dat"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/119.0.0.0 Safari/537.36"
    }
    r = requests.get(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["ContractBaseInfo"])
    temp_df.rename(
        columns={
            "BASISPRICE": "挂牌基准价",
            "ENDDELIVDATE": "最后交割日",
            "EXPIREDATE": "到期日",
            "INSTRUMENTID": "合约代码",
            "OPENDATE": "上市日",
            "STARTDELIVDATE": "开始交割日",
            "TRADINGDAY": "交易日",
            "UPDATE_DATE": "更新时间",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "合约代码",
            "上市日",
            "到期日",
            "开始交割日",
            "最后交割日",
            "挂牌基准价",
            "交易日",
            "更新时间",
        ]
    ]
    temp_df["上市日"] = pd.to_datetime(temp_df["上市日"], errors="coerce").dt.date
    temp_df["到期日"] = pd.to_datetime(temp_df["到期日"], errors="coerce").dt.date
    temp_df["开始交割日"] = pd.to_datetime(
        temp_df["开始交割日"], errors="coerce"
    ).dt.date
    temp_df["最后交割日"] = pd.to_datetime(
        temp_df["最后交割日"], errors="coerce"
    ).dt.date
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    temp_df["挂牌基准价"] = pd.to_numeric(temp_df["挂牌基准价"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_contract_info_shfe_df = futures_contract_info_shfe(date="20240513")
    print(futures_contract_info_shfe_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/8/2 16:47
Desc: 运动-奥运会
https://www.kaggle.com/marcogdepinto/let-s-discover-more-about-the-olympic-games
"""

import pandas as pd


def sport_olympic_hist() -> pd.DataFrame:
    """
    运动-奥运会-奖牌数据
    https://www.kaggle.com/marcogdepinto/let-s-discover-more-about-the-olympic-games
    :return: 奥运会-奖牌数据
    :rtype: pandas.DataFrame
    """
    url = "https://jfds-1252952517.cos.ap-chengdu.myqcloud.com/akshare/data/data_olympic/athlete_events.zip"
    temp_df = pd.read_csv(url)
    columns_list = [item.lower() for item in temp_df.columns.tolist()]
    temp_df.columns = columns_list
    return temp_df


if __name__ == "__main__":
    sport_olympic_hist_df = sport_olympic_hist()
    print(sport_olympic_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/12/18 17:30
Desc: 东方财富网-数据中心-特色数据-股权质押
东方财富网-数据中心-特色数据-股权质押-股权质押市场概况: https://data.eastmoney.com/gpzy/marketProfile.aspx
东方财富网-数据中心-特色数据-股权质押-上市公司质押比例: https://data.eastmoney.com/gpzy/pledgeRatio.aspx
东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细: https://data.eastmoney.com/gpzy/pledgeDetail.aspx
东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-证券公司: https://data.eastmoney.com/gpzy/distributeStatistics.aspx
东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-银行: https://data.eastmoney.com/gpzy/distributeStatistics.aspx
东方财富网-数据中心-特色数据-股权质押-行业数据: https://data.eastmoney.com/gpzy/industryData.aspx
"""

import math

import pandas as pd
import requests
from akshare.utils.tqdm import get_tqdm


def stock_gpzy_profile_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-股权质押市场概况
    https://data.eastmoney.com/gpzy/marketProfile.aspx
    :return: 股权质押市场概况
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_CSDC_STATISTICS",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in range(1, total_page + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.columns = [
        "交易日期",
        "质押总股数",
        "质押总市值",
        "沪深300指数",
        "涨跌幅",
        "A股质押总比例",
        "质押公司数量",
        "质押笔数",
    ]
    big_df = big_df[
        [
            "交易日期",
            "A股质押总比例",
            "质押公司数量",
            "质押笔数",
            "质押总股数",
            "质押总市值",
            "沪深300指数",
            "涨跌幅",
        ]
    ]
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    big_df["A股质押总比例"] = pd.to_numeric(big_df["A股质押总比例"], errors="coerce")
    big_df["质押公司数量"] = pd.to_numeric(big_df["质押公司数量"], errors="coerce")
    big_df["质押笔数"] = pd.to_numeric(big_df["质押笔数"], errors="coerce")
    big_df["质押总股数"] = pd.to_numeric(big_df["质押总股数"], errors="coerce")
    big_df["质押总市值"] = pd.to_numeric(big_df["质押总市值"], errors="coerce")
    big_df["沪深300指数"] = pd.to_numeric(big_df["沪深300指数"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["A股质押总比例"] = big_df["A股质押总比例"] / 100
    big_df["A股质押总比例"] = pd.to_numeric(big_df["A股质押总比例"], errors="coerce")
    big_df.sort_values(["交易日期"], inplace=True)
    big_df.reset_index(inplace=True, drop=True)
    return big_df


def stock_gpzy_pledge_ratio_em(date: str = "20240906") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-上市公司质押比例
    https://data.eastmoney.com/gpzy/pledgeRatio.aspx
    :param date: 指定交易日, 访问 https://data.eastmoney.com/gpzy/pledgeRatio.aspx 查询
    :type date: str
    :return: 上市公司质押比例
    :rtype: pandas.DataFrame
    """
    trade_date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "PLEDGE_RATIO",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_CSDC_LIST",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
        "filter": f"(TRADE_DATE='{trade_date}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "-",
        "股票代码",
        "股票简称",
        "交易日期",
        "质押比例",
        "质押股数",
        "质押笔数",
        "无限售股质押数",
        "限售股质押数",
        "质押市值",
        "所属行业",
        "近一年涨跌幅",
        "所属行业代码",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "交易日期",
            "所属行业",
            "质押比例",
            "质押股数",
            "质押市值",
            "质押笔数",
            "无限售股质押数",
            "限售股质押数",
            "近一年涨跌幅",
            "所属行业代码",
        ]
    ]
    big_df["质押比例"] = pd.to_numeric(big_df["质押比例"], errors="coerce")
    big_df["质押股数"] = pd.to_numeric(big_df["质押股数"], errors="coerce")
    big_df["质押市值"] = pd.to_numeric(big_df["质押市值"], errors="coerce")
    big_df["质押笔数"] = pd.to_numeric(big_df["质押笔数"], errors="coerce")
    big_df["无限售股质押数"] = pd.to_numeric(big_df["无限售股质押数"], errors="coerce")
    big_df["限售股质押数"] = pd.to_numeric(big_df["限售股质押数"], errors="coerce")
    big_df["近一年涨跌幅"] = pd.to_numeric(big_df["近一年涨跌幅"], errors="coerce")
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    return big_df


def _get_page_num_gpzy_market_pledge_ratio_detail() -> int:
    """
    东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细
    https://data.eastmoney.com/gpzy/pledgeDetail.aspx
    :return: int 获取 重要股东股权质押明细 的总页数
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPTA_APP_ACCUMDETAILS",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = math.ceil(int(data_json["result"]["count"]) / 500)
    return total_page


def stock_gpzy_pledge_ratio_detail_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细
    https://data.eastmoney.com/gpzy/pledgeDetail.aspx
    :return: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    total_page = _get_page_num_gpzy_market_pledge_ratio_detail()
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params = {
            "sortColumns": "NOTICE_DATE",
            "sortTypes": "-1",
            "pageSize": "500",
            "pageNumber": page,
            "reportName": "RPTA_APP_ACCUMDETAILS",
            "columns": "ALL",
            "quoteColumns": "",
            "source": "WEB",
            "client": "WEB",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "股票简称",
        "_",
        "股票代码",
        "股东名称",
        "_",
        "_",
        "_",
        "公告日期",
        "质押机构",
        "质押股份数量",
        "占所持股份比例",
        "占总股本比例",
        "质押日收盘价",
        "质押开始日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "预估平仓线",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "最新价",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "股东名称",
            "质押股份数量",
            "占所持股份比例",
            "占总股本比例",
            "质押机构",
            "最新价",
            "质押日收盘价",
            "预估平仓线",
            "质押开始日期",
            "公告日期",
        ]
    ]

    big_df["质押股份数量"] = pd.to_numeric(big_df["质押股份数量"], errors="coerce")
    big_df["占所持股份比例"] = pd.to_numeric(big_df["占所持股份比例"], errors="coerce")
    big_df["占总股本比例"] = pd.to_numeric(big_df["占总股本比例"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["质押日收盘价"] = pd.to_numeric(big_df["质押日收盘价"], errors="coerce")
    big_df["预估平仓线"] = pd.to_numeric(big_df["预估平仓线"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["质押开始日期"] = pd.to_datetime(
        big_df["质押开始日期"], errors="coerce"
    ).dt.date
    return big_df


def stock_gpzy_distribute_statistics_company_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-证券公司
    https://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: 质押机构分布统计-证券公司
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "ORG_NUM",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_GDZY_ZYJG_SUM",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
        "filter": '(PFORG_TYPE="证券")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "_",
        "_",
        "_",
        "_",
        "质押机构",
        "_",
        "质押公司数量",
        "质押笔数",
        "质押数量",
        "未达预警线比例",
        "达到预警线未达平仓线比例",
        "达到平仓线比例",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "质押机构",
            "质押公司数量",
            "质押笔数",
            "质押数量",
            "未达预警线比例",
            "达到预警线未达平仓线比例",
            "达到平仓线比例",
        ]
    ]
    temp_df["质押公司数量"] = pd.to_numeric(temp_df["质押公司数量"], errors="coerce")
    temp_df["质押笔数"] = pd.to_numeric(temp_df["质押笔数"], errors="coerce")
    temp_df["质押数量"] = pd.to_numeric(temp_df["质押数量"], errors="coerce")
    temp_df["未达预警线比例"] = pd.to_numeric(
        temp_df["未达预警线比例"], errors="coerce"
    )
    temp_df["达到预警线未达平仓线比例"] = pd.to_numeric(
        temp_df["达到预警线未达平仓线比例"], errors="coerce"
    )
    temp_df["达到平仓线比例"] = pd.to_numeric(
        temp_df["达到平仓线比例"], errors="coerce"
    )
    return temp_df


def stock_gpzy_distribute_statistics_bank_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-银行
    https://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: 质押机构分布统计-银行
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "ORG_NUM",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_GDZY_ZYJG_SUM",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
        "filter": '(PFORG_TYPE="银行")',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "_",
        "_",
        "_",
        "_",
        "质押机构",
        "_",
        "质押公司数量",
        "质押笔数",
        "质押数量",
        "未达预警线比例",
        "达到预警线未达平仓线比例",
        "达到平仓线比例",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "质押机构",
            "质押公司数量",
            "质押笔数",
            "质押数量",
            "未达预警线比例",
            "达到预警线未达平仓线比例",
            "达到平仓线比例",
        ]
    ]
    temp_df["质押公司数量"] = pd.to_numeric(temp_df["质押公司数量"], errors="coerce")
    temp_df["质押笔数"] = pd.to_numeric(temp_df["质押笔数"], errors="coerce")
    temp_df["质押数量"] = pd.to_numeric(temp_df["质押数量"], errors="coerce")
    temp_df["未达预警线比例"] = pd.to_numeric(
        temp_df["未达预警线比例"], errors="coerce"
    )
    temp_df["达到预警线未达平仓线比例"] = pd.to_numeric(
        temp_df["达到预警线未达平仓线比例"], errors="coerce"
    )
    temp_df["达到平仓线比例"] = pd.to_numeric(
        temp_df["达到平仓线比例"], errors="coerce"
    )
    return temp_df


def stock_gpzy_industry_data_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-上市公司质押比例-行业数据
    https://data.eastmoney.com/gpzy/industryData.aspx
    :return: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "AVERAGE_PLEDGE_RATIO",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_CSDC_INDUSTRY_STATISTICS",
        "columns": "INDUSTRY_CODE,INDUSTRY,TRADE_DATE,AVERAGE_PLEDGE_RATIO,ORG_NUM,PLEDGE_TOTAL_NUM,"
        "TOTAL_PLEDGE_SHARES,PLEDGE_TOTAL_MARKETCAP",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.columns = [
        "序号",
        "-",
        "行业",
        "统计时间",
        "平均质押比例",
        "公司家数",
        "质押总笔数",
        "质押总股本",
        "最新质押市值",
    ]
    temp_df = temp_df[
        [
            "序号",
            "行业",
            "平均质押比例",
            "公司家数",
            "质押总笔数",
            "质押总股本",
            "最新质押市值",
            "统计时间",
        ]
    ]
    temp_df["统计时间"] = pd.to_datetime(temp_df["统计时间"], errors="coerce").dt.date
    temp_df["平均质押比例"] = pd.to_numeric(temp_df["平均质押比例"], errors="coerce")
    temp_df["公司家数"] = pd.to_numeric(temp_df["公司家数"], errors="coerce")
    temp_df["质押总笔数"] = pd.to_numeric(temp_df["质押总笔数"], errors="coerce")
    temp_df["质押总股本"] = pd.to_numeric(temp_df["质押总股本"], errors="coerce")
    temp_df["最新质押市值"] = pd.to_numeric(temp_df["最新质押市值"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_gpzy_profile_em_df = stock_gpzy_profile_em()
    print(stock_gpzy_profile_em_df)

    stock_em_gpzy_pledge_ratio_df = stock_gpzy_pledge_ratio_em(date="20241220")
    print(stock_em_gpzy_pledge_ratio_df)

    stock_gpzy_pledge_ratio_detail_em_df = stock_gpzy_pledge_ratio_detail_em()
    print(stock_gpzy_pledge_ratio_detail_em_df)

    stock_em_gpzy_distribute_statistics_company_df = (
        stock_gpzy_distribute_statistics_company_em()
    )
    print(stock_em_gpzy_distribute_statistics_company_df)

    stock_em_gpzy_distribute_statistics_bank_df = (
        stock_gpzy_distribute_statistics_bank_em()
    )
    print(stock_em_gpzy_distribute_statistics_bank_df)

    stock_gpzy_industry_data_em_df = stock_gpzy_industry_data_em()
    print(stock_gpzy_industry_data_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/27 14:00
Desc: 美股-基本面数据
https://emweb.eastmoney.com/PC_USF10/pages/index.html?code=TSLA&type=web&color=w#/cwfx
"""

import pandas as pd
import requests

from akshare.utils.cons import headers


def __stock_financial_us_report_query_market_em(symbol: str = "TSLA") -> str:
    """
    东方财富-美股-财务分析-三大报表-查询市场
    https://emweb.eastmoney.com/PC_USF10/pages/index.html?code=TSLA&type=web&color=w#/cwfx
    :param symbol: choice of {"资产负债表", "综合损益表", "现金流量表"}
    :type symbol: str
    :return: 查询市场
    :rtype: str
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_USF10_INFO_ORGPROFILE",
        "columns": "SECUCODE,SECURITY_CODE,ORG_CODE,SECURITY_INNER_CODE,ORG_NAME,ORG_EN_ABBR,BELONG_INDUSTRY,"
        "FOUND_DATE,CHAIRMAN,REG_PLACE,ADDRESS,EMP_NUM,ORG_TEL,ORG_FAX,ORG_EMAIL,ORG_WEB,ORG_PROFILE",
        "quoteColumns": "",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "pageNumber": "1",
        "pageSize": "200",
        "sortTypes": "",
        "sortColumns": "",
        "source": "SECURITIES",
        "client": "PC",
        "v": "04406064331266868",
    }

    r = requests.get(url, params=params)
    data_json = r.json()
    stock_code = data_json["result"]["data"][0]["SECUCODE"]
    return stock_code


def __stock_financial_us_report_em(
    stock: str = "TSLA", symbol: str = "综合损益表", indicator: str = "年报"
) -> str:
    """
    东方财富-美股-财务分析-三大报表
    https://emweb.eastmoney.com/PC_USF10/pages/index.html?code=TSLA&type=web&color=w#/cwfx
    :param stock: 股票代码
    :type stock: str
    :param symbol: choice of {"资产负债表", "综合损益表", "现金流量表"}
    :type symbol: str
    :param indicator: choice of {"年报", "单季报", "累计季报"}
    :type indicator: str
    :return: 东方财富-美股-财务分析-三大报表
    :rtype: str
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    stock = __stock_financial_us_report_query_market_em(stock)
    if symbol == "资产负债表":
        report_name = "RPT_USF10_FN_BALANCE"
    elif symbol == "综合损益表":
        report_name = "RPT_USF10_FN_INCOME"
    elif symbol == "现金流量表":
        report_name = "RPT_USSK_FN_CASHFLOW"
    else:
        raise ValueError("请输入正确的 symbol 参数")
    params = {
        "reportName": report_name,
        "columns": "SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,REPORT,REPORT_DATE,FISCAL_YEAR,CURRENCY,"
        "ACCOUNT_STANDARD,REPORT_TYPE,DATE_TYPE_CODE",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{stock}")',
        "pageNumber": "",
        "pageSize": "",
        "sortTypes": "-1",
        "sortColumns": "REPORT_DATE",
        "source": "SECURITIES",
        "client": "PC",
        "v": "09583551779242467",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_tuple = tuple(set(temp_df["REPORT"].tolist()))
    if indicator == "年报":
        tuple_data = tuple(item.strip() for item in temp_tuple if "FY" in item)
    elif indicator == "单季报":
        tuple_data = tuple(item.strip() for item in temp_tuple if "Q1" in item)
    elif indicator == "累计季报":
        tuple_data = tuple(
            item.strip() for item in temp_tuple if "Q6" in item or "Q9" in item
        )
    else:
        raise ValueError("请输入正确的 indicator 参数")
    sorted_tuple = tuple(
        sorted(tuple_data, key=lambda x: x.split("/")[0], reverse=True)
    )
    double_quotes_str = str(sorted_tuple).replace("'", '"')
    double_quotes_str = double_quotes_str.replace(" ", "")
    return double_quotes_str


def stock_financial_us_report_em(
    stock: str = "TSLA", symbol: str = "资产负债表", indicator: str = "年报"
) -> pd.DataFrame:
    """
    东方财富-美股-财务分析-三大报表
    https://emweb.eastmoney.com/PC_USF10/pages/index.html?code=TSLA&type=web&color=w#/cwfx
    :param stock: 股票代码
    :type stock: str
    :param symbol: choice of {"资产负债表", "综合损益表", "现金流量表"}
    :type symbol: str
    :param indicator: choice of {"年报", "单季报", "累计季报"}
    :type indicator: str
    :return: 东方财富-美股-财务分析-三大报表
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    date_str = __stock_financial_us_report_em(
        stock=stock, symbol=symbol, indicator=indicator
    )
    stock = __stock_financial_us_report_query_market_em(stock)
    if symbol == "资产负债表":
        report_name = "RPT_USF10_FN_BALANCE"
    elif symbol == "综合损益表":
        report_name = "RPT_USF10_FN_INCOME"
    elif symbol == "现金流量表":
        report_name = "RPT_USSK_FN_CASHFLOW"
    else:
        raise ValueError("请输入正确的 symbol 参数")
    params = {
        "reportName": report_name,
        "columns": "SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,REPORT_DATE,REPORT_TYPE,REPORT,"
        "STD_ITEM_CODE,AMOUNT,ITEM_NAME",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{stock}")(REPORT in ' + date_str + ")",
        "pageNumber": "",
        "pageSize": "",
        "sortTypes": "1,-1",
        "sortColumns": "STD_ITEM_CODE,REPORT_DATE",
        "source": "SECURITIES",
        "client": "PC",
        "v": "09583551779242467",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    return temp_df


def stock_financial_us_analysis_indicator_em(
    symbol: str = "TSLA", indicator: str = "年报"
) -> pd.DataFrame:
    """
    东方财富-美股-财务分析-主要指标
    https://emweb.eastmoney.com/PC_USF10/pages/index.html?code=TSLA&type=web&color=w#/cwfx
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: choice of {"年报", "单季报", "累计季报"}
    :type indicator: str
    :return: 东方财富-美股-财务分析-主要指标
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    symbol = __stock_financial_us_report_query_market_em(symbol)
    params = {
        "reportName": "RPT_USF10_FN_GMAININDICATOR",
        "columns": "USF10_FN_GMAININDICATOR",
        "quoteColumns": "",
        "pageNumber": "",
        "pageSize": "",
        "sortTypes": "-1",
        "sortColumns": "STD_REPORT_DATE",
        "source": "F10",
        "client": "PC",
        "v": "01975982096513973",
    }
    if indicator == "年报":
        params.update({"filter": f"""(SECUCODE="{symbol}")(DATE_TYPE_CODE="001")"""})
    elif indicator == "单季报":
        params.update(
            {
                "filter": f"""(SECUCODE="{symbol}")(DATE_TYPE_CODE in ("003","006","007","008"))"""
            }
        )
    elif indicator == "累计季报":
        params.update(
            {"filter": f"""(SECUCODE="{symbol}")(DATE_TYPE_CODE in ("002","004"))"""}
        )
    else:
        raise ValueError("请输入正确的 indicator 参数")
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    return temp_df


if __name__ == "__main__":
    stock_financial_us_analysis_indicator_em_df = (
        stock_financial_us_analysis_indicator_em(symbol="BABA", indicator="年报")
    )
    print(stock_financial_us_analysis_indicator_em_df)

    stock_financial_us_analysis_indicator_em_df = (
        stock_financial_us_analysis_indicator_em(symbol="TSLA", indicator="单季报")
    )
    print(stock_financial_us_analysis_indicator_em_df)

    stock_financial_us_analysis_indicator_em_df = (
        stock_financial_us_analysis_indicator_em(symbol="TSLA", indicator="累计季报")
    )
    print(stock_financial_us_analysis_indicator_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="BABA", symbol="资产负债表", indicator="年报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="资产负债表", indicator="单季报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="资产负债表", indicator="累计季报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="综合损益表", indicator="年报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="综合损益表", indicator="单季报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="综合损益表", indicator="累计季报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="现金流量表", indicator="年报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="现金流量表", indicator="单季报"
    )
    print(stock_financial_us_report_em_df)

    stock_financial_us_report_em_df = stock_financial_us_report_em(
        stock="TSLA", symbol="现金流量表", indicator="累计季报"
    )
    print(stock_financial_us_report_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/14 18:00
Desc: 国证指数
http://www.cnindex.com.cn/index.html
"""

import zipfile
from io import BytesIO

import pandas as pd
import requests


def index_all_cni() -> pd.DataFrame:
    """
    国证指数-最近交易日的所有指数
    http://www.cnindex.com.cn/zh_indices/sese/index.html?act_menu=1&index_type=-1
    :return: 国证指数-所有指数
    :rtype: pandas.DataFrame
    """
    url = "http://www.cnindex.com.cn/index/indexList"
    params = {
        "channelCode": "-1",
        "rows": "2000",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["rows"])
    temp_df.columns = [
        "_",
        "_",
        "指数代码",
        "_",
        "_",
        "_",
        "_",
        "_",
        "指数简称",
        "_",
        "_",
        "_",
        "样本数",
        "收盘点位",
        "涨跌幅",
        "_",
        "PE滚动",
        "_",
        "成交量",
        "成交额",
        "总市值",
        "自由流通市值",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "指数代码",
            "指数简称",
            "样本数",
            "收盘点位",
            "涨跌幅",
            "PE滚动",
            "成交量",
            "成交额",
            "总市值",
            "自由流通市值",
        ]
    ]
    temp_df["成交量"] = temp_df["成交量"] / 100000
    temp_df["成交额"] = temp_df["成交额"] / 100000000
    temp_df["总市值"] = temp_df["总市值"] / 100000000
    temp_df["自由流通市值"] = temp_df["自由流通市值"] / 100000000
    return temp_df


def index_hist_cni(
    symbol: str = "399001", start_date: str = "20230114", end_date: str = "20240114"
) -> pd.DataFrame:
    """
    指数历史行情数据
    http://www.cnindex.com.cn/module/index-detail.html?act_menu=1&indexCode=399001
    :param symbol: 指数代码
    :type symbol: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 指数历史行情数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "http://hq.cnindex.com.cn/market/market/getIndexDailyDataWithDataFormat"
    params = {
        "indexCode": symbol,
        "startDate": start_date,
        "endDate": end_date,
        "frequency": "day",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["data"])
    temp_df.columns = [
        "日期",
        "_",
        "最高价",
        "开盘价",
        "最低价",
        "收盘价",
        "_",
        "涨跌幅",
        "成交额",
        "成交量",
        "_",
    ]
    temp_df = temp_df[
        [
            "日期",
            "开盘价",
            "最高价",
            "最低价",
            "收盘价",
            "涨跌幅",
            "成交量",
            "成交额",
        ]
    ]
    temp_df["涨跌幅"] = temp_df["涨跌幅"].str.replace("%", "")
    temp_df["涨跌幅"] = temp_df["涨跌幅"].astype("float")
    temp_df["涨跌幅"] = temp_df["涨跌幅"] / 100
    temp_df.sort_values(["日期"], inplace=True, ignore_index=True)
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["开盘价"] = pd.to_numeric(temp_df["开盘价"], errors="coerce")
    temp_df["最高价"] = pd.to_numeric(temp_df["最高价"], errors="coerce")
    temp_df["最低价"] = pd.to_numeric(temp_df["最低价"], errors="coerce")
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def index_detail_cni(symbol: str = "399001", date: str = "202404") -> pd.DataFrame:
    """
    国证指数-样本详情-指定日期的样本成份
    http://www.cnindex.com.cn/module/index-detail.html?act_menu=1&indexCode=399001
    :param symbol: 指数代码
    :type symbol: str
    :param date: 指定月份
    :type date: str
    :return: 指定日期的样本成份
    :rtype: pandas.DataFrame
    """
    url = "http://www.cnindex.com.cn/sample-detail/download"
    params = {"indexcode": symbol, "dateStr": "-".join([date[:4], date[4:]])}
    r = requests.get(url, params=params)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df["样本代码"] = temp_df["样本代码"].astype(str).str.zfill(6)
    temp_df.columns = [
        "日期",
        "样本代码",
        "样本简称",
        "所属行业",
        "总市值",
        "权重",
    ]
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["权重"] = pd.to_numeric(temp_df["权重"], errors="coerce")
    return temp_df


def index_detail_hist_cni(symbol: str = "399001", date: str = "") -> pd.DataFrame:
    """
    国证指数-样本详情-历史样本
    http://www.cnindex.com.cn/module/index-detail.html?act_menu=1&indexCode=399001
    :param symbol: 指数代码; "399001"
    :type symbol: str
    :param date: 指定月份; "202201", 为空返回所有数据
    :type date: str
    :return: 历史样本
    :rtype: pandas.DataFrame
    """
    if date:
        url = "http://www.cnindex.com.cn/sample-detail/detail"
        params = {
            "indexcode": symbol,
            "dateStr": "-".join([date[:4], date[4:]]),
            "pageNum": "1",
            "rows": "50000",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"]["rows"])
        temp_df.columns = [
            "-",
            "-",
            "日期",
            "样本代码",
            "样本简称",
            "所属行业",
            "-",
            "总市值",
            "权重",
            "-",
        ]
        temp_df = temp_df[
            [
                "日期",
                "样本代码",
                "样本简称",
                "所属行业",
                "总市值",
                "权重",
            ]
        ]
    else:
        url = "http://www.cnindex.com.cn/sample-detail/download-history"
        params = {"indexcode": symbol}
        r = requests.get(url, params=params)
        temp_df = pd.read_excel(BytesIO(r.content))
    temp_df["样本代码"] = temp_df["样本代码"].astype(str).str.zfill(6)
    temp_df.columns = [
        "日期",
        "样本代码",
        "样本简称",
        "所属行业",
        "总市值",
        "权重",
    ]
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"])
    temp_df["权重"] = pd.to_numeric(temp_df["权重"])
    return temp_df


def index_detail_hist_adjust_cni(symbol: str = "399005") -> pd.DataFrame:
    """
    国证指数-样本详情-历史调样
    http://www.cnindex.com.cn/module/index-detail.html?act_menu=1&indexCode=399005
    :param symbol: 指数代码
    :type symbol: str
    :return: 历史调样
    :rtype: pandas.DataFrame
    """
    url = "http://www.cnindex.com.cn/sample-detail/download-adjustment"
    params = {"indexcode": symbol}
    r = requests.get(url, params=params)
    try:
        import warnings

        with warnings.catch_warnings():
            warnings.simplefilter(action="ignore", category=UserWarning)
            temp_df = pd.read_excel(BytesIO(r.content), engine="openpyxl")
    except zipfile.BadZipFile:
        return pd.DataFrame()
    temp_df["样本代码"] = temp_df["样本代码"].astype(str).str.zfill(6)
    return temp_df


if __name__ == "__main__":
    index_all_cni_df = index_all_cni()
    print(index_all_cni_df)

    index_hist_cni_df = index_hist_cni(
        symbol="399005", start_date="20230114", end_date="20240114"
    )
    print(index_hist_cni_df)

    index_detail_cni_df = index_detail_cni(symbol="399001", date="202404")
    print(index_detail_cni_df)

    index_detail_hist_cni_df = index_detail_hist_cni(symbol="399101", date="202404")
    print(index_detail_hist_cni_df)

    index_detail_hist_adjust_cni_df = index_detail_hist_adjust_cni(symbol="399005")
    print(index_detail_hist_adjust_cni_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/2/20 17:00
Desc: 东方财富网-行情中心-美股市场-粉单市场
https://quote.eastmoney.com/center/gridlist.html#us_pinksheet
"""

import pandas as pd
import requests


def stock_us_pink_spot_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-美股市场-粉单市场
    https://quote.eastmoney.com/center/gridlist.html#us_pinksheet
    :return: 粉单市场实时行情
    :rtype: pandas.DataFrame
    """
    url = "https://23.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "50000",
        "po": "1",
        "np": "2",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:153",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,"
        "f26,f22,f33,f11,f62,f128,f136,f115,f152",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"]).T
    temp_df.columns = [
        "_",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "简称",
        "编码",
        "名称",
        "最高价",
        "最低价",
        "开盘价",
        "昨收价",
        "总市值",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "市盈率",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df) + 1)
    temp_df.rename(columns={"index": "序号"}, inplace=True)
    temp_df["代码"] = temp_df["编码"].astype(str) + "." + temp_df["简称"]
    temp_df = temp_df[
        [
            "序号",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "开盘价",
            "最高价",
            "最低价",
            "昨收价",
            "总市值",
            "市盈率",
            "代码",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["开盘价"] = pd.to_numeric(temp_df["开盘价"], errors="coerce")
    temp_df["最高价"] = pd.to_numeric(temp_df["最高价"], errors="coerce")
    temp_df["最低价"] = pd.to_numeric(temp_df["最低价"], errors="coerce")
    temp_df["昨收价"] = pd.to_numeric(temp_df["昨收价"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["市盈率"] = pd.to_numeric(temp_df["市盈率"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_us_pink_spot_em_df = stock_us_pink_spot_em()
    print(stock_us_pink_spot_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/2/4 23:00
Desc: 中国银行保险监督管理委员会-首页-政务信息-行政处罚-银保监分局本级-XXXX行政处罚信息公开表
https://www.nfra.gov.cn/cn/view/pages/ItemList.html?itemPId=923&itemId=4115&itemUrl=ItemListRightList.html&itemName=%E9%93%B6%E4%BF%9D%E7%9B%91%E5%88%86%E5%B1%80%E6%9C%AC%E7%BA%A7&itemsubPId=931&itemsubPName=%E8%A1%8C%E6%94%BF%E5%A4%84%E7%BD%9A#2
提取 具体页面 html 页面的 json 接口
https://www.nfra.gov.cn/cn/static/data/DocInfo/SelectByDocId/data_docId=881446.json
2020 新接口
"""

import warnings
from io import StringIO

import pandas as pd
import requests
from tqdm import tqdm

from akshare.bank.cons import cbirc_headers_without_cookie_2020


def bank_fjcf_total_num(item: str = "分局本级") -> int:
    """
    首页-政务信息-行政处罚-银保监分局本级 总页数
    https://www.nfra.gov.cn/cn/view/pages/ItemList.html?itemPId=923&itemId=4115&itemUrl=ItemListRightList.html&itemName=%E9%93%B6%E4%BF%9D%E7%9B%91%E5%88%86%E5%B1%80%E6%9C%AC%E7%BA%A7&itemsubPId=931
    :param item: choice of {"机关", "本级", "分局本级"}
    :type item: str
    :return: 总页数
    :rtype: int
    """
    item_id_list = {
        "机关": "4113",
        "本级": "4114",
        "分局本级": "4115",
    }
    cbirc_headers = cbirc_headers_without_cookie_2020.copy()
    main_url = "https://www.nfra.gov.cn/cbircweb/DocInfo/SelectDocByItemIdAndChild"
    params = {
        "itemId": item_id_list[item],
        "pageSize": "18",
        "pageIndex": "1",
    }
    res = requests.get(main_url, params=params, headers=cbirc_headers)
    return int(res.json()["data"]["total"])


def bank_fjcf_total_page(item: str = "分局本级", begin: int = 1) -> int:
    """
    获取首页-政务信息-行政处罚-银保监分局本级的总页数
    https://www.nfra.gov.cn/cn/view/pages/ItemList.html?itemPId=923&itemId=4115&itemUrl=ItemListRightList.html&itemName=%E9%93%B6%E4%BF%9D%E7%9B%91%E5%88%86%E5%B1%80%E6%9C%AC%E7%BA%A7&itemsubPId=931
    :param item: choice of {"机关", "本级", "分局本级"}
    :type item: str
    :param begin: 开始页数
    :type begin: str
    :return: 总页数
    :rtype: int
    """
    item_id_list = {
        "机关": "4113",
        "本级": "4114",
        "分局本级": "4115",
    }
    cbirc_headers = cbirc_headers_without_cookie_2020.copy()
    main_url = "https://www.nfra.gov.cn/cbircweb/DocInfo/SelectDocByItemIdAndChild"
    params = {
        "itemId": item_id_list[item],
        "pageSize": "18",
        "pageIndex": str(begin),
    }
    res = requests.get(main_url, params=params, headers=cbirc_headers)
    if res.json()["data"]["total"] / 18 > int(res.json()["data"]["total"] / 18):
        total_page = int(res.json()["data"]["total"] / 18) + 1
        return total_page


def bank_fjcf_page_url(
    page: int = 5, item: str = "分局本级", begin: int = 1
) -> pd.DataFrame:
    """
    获取 首页-政务信息-行政处罚-银保监分局本级-每一页的 json 数据
    :param page: 需要获取前 page 页的内容, 总页数请通过 ak.bank_fjcf_total_page() 获取
    :type page: int
    :param item: choice of {"机关", "本级", "分局本级"}
    :type item: str
    :param begin: 开始页数
    :type begin: str
    :return: 需要的字段
    :rtype: pandas.DataFrame
    """
    item_id_list = {
        "机关": "4113",
        "本级": "4114",
        "分局本级": "4115",
    }
    cbirc_headers = cbirc_headers_without_cookie_2020.copy()
    main_url = "https://www.nfra.gov.cn/cbircweb/DocInfo/SelectDocByItemIdAndChild"
    temp_df = pd.DataFrame()
    for i_page in tqdm(range(begin, page + begin), leave=False):
        params = {
            "itemId": item_id_list[item],
            "pageSize": "18",
            "pageIndex": str(i_page),
        }
        res = requests.get(main_url, params=params, headers=cbirc_headers)
        temp_df = pd.concat([temp_df, pd.DataFrame(res.json()["data"]["rows"])])
    return temp_df[
        ["docId", "docSubtitle", "publishDate", "docFileUrl", "docTitle", "generaltype"]
    ]


def bank_fjcf_table_detail(
    page: int = 5, item: str = "分局本级", begin: int = 1
) -> pd.DataFrame:
    """
    获取 首页-政务信息-行政处罚-银保监分局本级-XXXX行政处罚信息公开表 数据
    :param page: 需要获取前 page 页的内容, 总页数请通过 ak.bank_fjcf_total_page() 获取
    :type page: int
    :param item: choice of {"机关", "本级", "分局本级"}
    :type item: str
    :param begin: 开始页面
    :type begin: int
    :return: 返回所有行政处罚信息公开表的集合, 按第一页到最后一页的顺序排列
    :rtype: pandas.DataFrame
    """
    id_list = bank_fjcf_page_url(page=page, item=item, begin=begin)["docId"]
    big_df = pd.DataFrame()
    for item in id_list:
        url = f"https://www.nfra.gov.cn/cn/static/data/DocInfo/SelectByDocId/data_docId={item}.json"
        res = requests.get(url)
        try:
            table_list = pd.read_html(StringIO(res.json()["data"]["docClob"]))[0]
            if table_list.shape[1] == 2:
                table_list = table_list.iloc[:, 1].values.tolist()
            else:
                table_list = table_list.iloc[:, 3:].values.tolist()
            # 部分旧表缺少字段，所以填充
            if len(table_list) == 7:
                table_list.insert(2, pd.NA)
                table_list.insert(3, pd.NA)
                table_list.insert(4, pd.NA)
            elif len(table_list) == 8:
                table_list.insert(1, pd.NA)
                table_list.insert(2, pd.NA)
            elif len(table_list) == 9:
                table_list.insert(2, pd.NA)
            elif len(table_list) == 11:
                table_list = table_list[2:]
                table_list.insert(2, pd.NA)
            else:
                print(
                    f"{item} 异常，请通过 https://www.nfra.gov.cn/cn/view/pages/ItemDetail.html?docId={item} 查看"
                )
                continue

            # 部分会变成嵌套列表, 这里还原
            table_list = [
                item[0] if isinstance(item, list) else item for item in table_list
            ]
            table_list.append(str(item))
            table_list.append(res.json()["data"]["publishDate"])
            table_df = pd.DataFrame(table_list)
            table_df.columns = ["内容"]
            big_df = pd.concat(objs=[big_df, table_df.T], ignore_index=True)
            # 解决有些页面缺少字段的问题, 都放到 try 里面
        except:  # noqa: E722
            warnings.warn(f"{item} 不是表格型数据，将跳过采集")
            continue
    if big_df.empty:
        return pd.DataFrame()
    big_df.columns = [
        "行政处罚决定书文号",
        "姓名",
        "单位",  # 20200108 新增
        "单位名称",
        "主要负责人姓名",
        "主要违法违规事实（案由）",
        "行政处罚依据",
        "行政处罚决定",
        "作出处罚决定的机关名称",
        "作出处罚决定的日期",
        "处罚ID",
        "处罚公布日期",
    ]
    return big_df


if __name__ == "__main__":
    bank_fjcf_table_detail_df = bank_fjcf_table_detail(page=1, item="机关", begin=1)
    print(bank_fjcf_table_detail_df)
